@@ -1,3 +1,6 @@
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <stdlib.h>
 #include <glib.h>
 #include <gtk/gtk.h>
@@ -10,7 +13,7 @@ static GList *seekable_elements = NULL;
 static GstElement *pipeline;
 static GtkAdjustment *adjustment;
 static gboolean stats = FALSE;
-static guint64 duration;
+static gint64 duration;
 
 static guint update_id;
 
@@ -37,12 +40,12 @@ make_spider_pipeline (const gchar * location, gboolean thread)
   decoder = gst_element_factory_make ("spider", "decoder");
   a_thread = gst_thread_new ("a_thread");
   a_queue = gst_element_factory_make ("queue", "a_queue");
-  audiosink = gst_element_factory_make ("osssink", "a_sink");
+  audiosink = gst_element_factory_make (DEFAULT_AUDIOSINK, "a_sink");
   //g_object_set (G_OBJECT (audiosink), "fragment", 0x00180008, NULL);
 
   v_thread = gst_thread_new ("v_thread");
   v_queue = gst_element_factory_make ("queue", "v_queue");
-  videosink = gst_element_factory_make ("xvideosink", "v_sink");
+  videosink = gst_element_factory_make (DEFAULT_VIDEOSINK, "v_sink");
   //g_object_set (G_OBJECT (audiosink), "sync", FALSE, NULL);
 
   g_object_set (G_OBJECT (src), "location", location, NULL);
@@ -133,7 +136,7 @@ query_rates (void)
 }
 
 G_GNUC_UNUSED static void
-query_durations ()
+query_durations (void)
 {
   GList *walk = seekable_elements;
 
@@ -162,7 +165,7 @@ query_durations ()
 }
 
 G_GNUC_UNUSED static void
-query_positions ()
+query_positions (void)
 {
   GList *walk = seekable_elements;
 
@@ -194,7 +197,7 @@ static gboolean
 update_scale (gpointer data)
 {
   GstClock *clock;
-  guint64 position;
+  gint64 position;
   GstFormat format = GST_FORMAT_TIME;
 
   duration = 0;
@@ -208,8 +211,8 @@ update_scale (gpointer data)
   position = gst_clock_get_time (clock);
 
   if (stats) {
-    g_print ("clock:                  %13llu  (%s)\n", position,
-        gst_object_get_name (GST_OBJECT (clock)));
+    g_print ("clock:                  %13" G_GUINT64_FORMAT "  (%s)\n",
+        position, gst_object_get_name (GST_OBJECT (clock)));
     query_durations ();
     query_positions ();
     query_rates ();
@@ -342,11 +345,11 @@ main (int argc, char **argv)
   gtk_scale_set_digits (GTK_SCALE (hscale), 2);
   gtk_range_set_update_policy (GTK_RANGE (hscale), GTK_UPDATE_CONTINUOUS);
 
-  gtk_signal_connect (GTK_OBJECT (hscale),
+  g_signal_connect (hscale,
       "button_press_event", G_CALLBACK (start_seek), pipeline);
-  gtk_signal_connect (GTK_OBJECT (hscale),
+  g_signal_connect (hscale,
       "button_release_event", G_CALLBACK (stop_seek), pipeline);
-  gtk_signal_connect (GTK_OBJECT (hscale),
+  g_signal_connect (hscale,
       "format_value", G_CALLBACK (format_value), pipeline);
 
   /* do the packing stuff ... */
