@@ -818,6 +818,39 @@ gst_alsa_caps (snd_pcm_format_t format, gint rate, gint channels)
   return ret_caps;
 }
 
+gboolean
+gst_alsa_set_default_format (GstAlsa * this)
+{
+  GstAlsaFormat *ret = NULL;
+
+  if (this->handle == NULL)
+    if (!gst_alsa_open_audio (this))
+      return FALSE;
+
+  /* Attempt to set a default format, fail if we can't */
+  ret = g_new (GstAlsaFormat, 1);
+  if (ret != NULL) {
+    ret->format = snd_pcm_build_linear_format (16, 16, 0, 0);
+    ret->rate = 44100;
+    ret->channels = 2;
+    if (gst_alsa_probe_hw_params (this, ret)) {
+      this->format = ret;
+
+      if (GST_FLAG_IS_SET (this, GST_ALSA_RUNNING))
+        gst_alsa_stop_audio (this);
+
+      if (gst_alsa_start_audio (this)) {
+        return TRUE;
+      }
+
+      this->format = NULL;
+    }
+    /* Give up too easily */
+    g_free (ret);
+  }
+
+  return FALSE;
+}
 
 static int
 gst_alsa_check_sample_rates (snd_pcm_t * device_handle,
@@ -1608,11 +1641,11 @@ gst_alsa_open_audio (GstAlsa * this)
       case ENXIO:
       case ENODEV:
       case ENOENT:
-        GST_ELEMENT_ERROR (GST_ELEMENT (this), RESOURCE, BUSY,
+        GST_ELEMENT_ERROR (GST_ELEMENT (this), RESOURCE, NOT_FOUND,
             (_("ALSA device \"%s\" does not exist."), this->device), (NULL));
         break;
       default:
-        GST_ELEMENT_ERROR (GST_ELEMENT (this), RESOURCE, BUSY,
+        GST_ELEMENT_ERROR (GST_ELEMENT (this), RESOURCE, OPEN_READ_WRITE,
             (_("ALSA device \"%s\" had an error."),
                 this->device), ("ALSA error %d: %s", ret, snd_strerror (ret)));
         break;
@@ -1786,8 +1819,7 @@ gst_alsa_set_hw_params (GstAlsa * this)
     period_size = this->period_size;
     ERROR_CHECK (snd_pcm_hw_params_set_period_size_near (this->handle,
             hw_params, &period_size, 0),
-        "error setting period size to %u frames: %s",
-        (guint) period_size);
+        "error setting period size to %u frames: %s", (guint) period_size);
     if (this->period_size == 0)
       this->period_size = period_size;
   } else {
