@@ -56,13 +56,16 @@ plugin_init (GstPlugin * plugin)
 {
   int err;
 
-  if (!gst_element_register (plugin, "alsamixer", GST_RANK_NONE,
+  if (!gst_library_load ("gstaudio"))
+    return FALSE;
+
+  if (!gst_element_register (plugin, "alsamixer", GST_RANK_PRIMARY,
           GST_TYPE_ALSA_MIXER))
     return FALSE;
-  if (!gst_element_register (plugin, "alsasrc", GST_RANK_NONE,
+  if (!gst_element_register (plugin, "alsasrc", GST_RANK_PRIMARY,
           GST_TYPE_ALSA_SRC))
     return FALSE;
-  if (!gst_element_register (plugin, "alsasink", GST_RANK_NONE,
+  if (!gst_element_register (plugin, "alsasink", GST_RANK_PRIMARY,
           GST_TYPE_ALSA_SINK))
     return FALSE;
 
