@@ -87,7 +87,7 @@ struct _CDParanoia {
   gboolean abort_on_skip;
   gint paranoia_mode;
   gchar *uri; /* URI caching */
-  gint uri_track;
+  gint cur_track;
   gint seek_request;
 
   cdrom_drive *d;
@@ -106,7 +106,9 @@ struct _CDParanoia {
   gint64 total_seconds;
 
   gint prev_sec;
-  gboolean discont_sent;
+  gboolean discont_sent, flush_pending;
+  gboolean track_mode;
+  GstEvent *pending_metadata;
 };
 
 struct _CDParanoiaClass {
