@@ -38,7 +38,6 @@
 #include <assert.h>
 
 #include "dvdreadsrc.h"
-#include "stream_labels.h"
 
 #include <dvdread/dvd_reader.h>
 #include <dvdread/ifo_types.h>
@@ -55,7 +54,7 @@ struct _DVDReadSrcPrivate
   gchar *location;
   gchar *last_uri;
 
-  gboolean new_seek;
+  gboolean new_seek, change_cell;
 
   gboolean new_cell;
 
@@ -74,6 +73,8 @@ struct _DVDReadSrcPrivate
   /* where we are */
   gboolean seek_pend, flush_pend;
   GstFormat seek_pend_fmt;
+  GstEvent *title_lang_event_pending;
+  GstEvent *pending_clut_event;
 };
 
 GST_DEBUG_CATEGORY_STATIC (gstdvdreadsrc_debug);
@@ -126,7 +127,6 @@ static GstElementStateReturn dvdreadsrc_change_state (GstElement * element);
 
 static void dvdreadsrc_uri_handler_init (gpointer g_iface, gpointer iface_data);
 
-
 static GstElementClass *parent_class = NULL;
 
 static GstFormat sector_format, angle_format, title_format, chapter_format;
@@ -245,6 +245,7 @@ dvdreadsrc_init (DVDReadSrc * dvdreadsrc)
   dvdreadsrc->priv->last_uri = NULL;
   dvdreadsrc->priv->new_seek = TRUE;
   dvdreadsrc->priv->new_cell = TRUE;
+  dvdreadsrc->priv->change_cell = FALSE;
   dvdreadsrc->priv->title = 0;
   dvdreadsrc->priv->chapter = 0;
   dvdreadsrc->priv->angle = 0;
@@ -252,6 +253,8 @@ dvdreadsrc_init (DVDReadSrc * dvdreadsrc)
   dvdreadsrc->priv->seek_pend = FALSE;
   dvdreadsrc->priv->flush_pend = FALSE;
   dvdreadsrc->priv->seek_pend_fmt = GST_FORMAT_UNDEFINED;
+  dvdreadsrc->priv->title_lang_event_pending = NULL;
+  dvdreadsrc->priv->pending_clut_event = NULL;
 }
 
 static void
@@ -379,14 +382,17 @@ dvdreadsrc_get_formats (GstPad * pad)
 {
   static GstFormat formats[] = {
     GST_FORMAT_BYTES,
+    GST_FORMAT_TIME,
     0, 0, 0, 0,                 /* init later */
     0,
   };
-  if (formats[1] == 0) {
-    formats[1] = sector_format;
-    formats[2] = angle_format;
-    formats[3] = title_format;
-    formats[4] = chapter_format;
+  const gint start = 2;
+
+  if (formats[start] == 0) {
+    formats[start] = sector_format;
+    formats[start + 1] = angle_format;
+    formats[start + 2] = title_format;
+    formats[start + 3] = chapter_format;
   }
 
   return formats;
@@ -484,6 +490,56 @@ dvdreadsrc_srcpad_event (GstPad * pad, GstEvent * event)
   return res;
 }
 
+static GstEvent *
+dvdnavsrc_make_clut_change_event (DVDReadSrcPrivate * src, const guint * clut)
+{
+  GstEvent *event;
+  GstStructure *structure;
+  gchar name[16];
+  int i;
+
+  structure = gst_structure_new ("application/x-gst-dvd",
+      "event", G_TYPE_STRING, "dvd-spu-clut-change", NULL);
+
+  /* Create a separate field for each value in the table. */
+  for (i = 0; i < 16; i++) {
+    sprintf (name, "clut%02d", i);
+    gst_structure_set (structure, name, G_TYPE_INT, (int) clut[i], NULL);
+  }
+
+  /* Create the DVD event and put the structure into it. */
+  event = gst_event_new (GST_EVENT_ANY);
+  event->event_data.structure.structure = structure;
+
+  return event;
+}
+
+static GstClockTime
+dvdreadsrc_convert_timecode (dvd_time_t * time)
+{
+  GstClockTime ret_time;
+  const GstClockTime one_hour = 3600 * GST_SECOND;
+  const GstClockTime one_min = 60 * GST_SECOND;
+
+  g_return_val_if_fail ((time->hour >> 4) < 0xa
+      && (time->hour & 0xf) < 0xa, GST_CLOCK_TIME_NONE);
+  g_return_val_if_fail ((time->minute >> 4) < 0x7
+      && (time->minute & 0xf) < 0xa, GST_CLOCK_TIME_NONE);
+  g_return_val_if_fail ((time->second >> 4) < 0x7
+      && (time->second & 0xf) < 0xa, GST_CLOCK_TIME_NONE);
+
+  ret_time =
+      (GstClockTime) ((time->hour >> 4) * 10 + (time->hour & 0xf)) * one_hour;
+  ret_time +=
+      (GstClockTime) ((time->minute >> 4) * 10 +
+      (time->minute & 0xf)) * one_min;
+  ret_time +=
+      (GstClockTime) ((time->second >> 4) * 10 +
+      (time->second & 0xf)) * GST_SECOND;
+
+  return ret_time;
+}
+
 static gboolean
 dvdreadsrc_srcpad_query (GstPad * pad, GstQueryType type,
     GstFormat * format, gint64 * value)
@@ -498,8 +554,18 @@ dvdreadsrc_srcpad_query (GstPad * pad, GstQueryType type,
   switch (type) {
     case GST_QUERY_TOTAL:
       switch (*format) {
+        case GST_FORMAT_TIME:
+          if (priv->cur_pgc != NULL) {
+            *value =
+                dvdreadsrc_convert_timecode (&priv->cur_pgc->playback_time);
+            if (*value == GST_CLOCK_TIME_NONE)
+              res = FALSE;
+          } else
+            res = FALSE;
+          break;
         case GST_FORMAT_BYTES:
-          *value = DVDFileSize (priv->dvd_title) * DVD_VIDEO_LB_LEN;
+          *value =
+              (GstClockTime) (DVDFileSize (priv->dvd_title)) * DVD_VIDEO_LB_LEN;
           break;
         default:
           if (*format == sector_format) {
@@ -511,7 +577,7 @@ dvdreadsrc_srcpad_query (GstPad * pad, GstQueryType type,
           } else if (*format == angle_format) {
             *value = priv->tt_srpt->title[priv->title].nr_of_angles;
           } else {
-            GST_LOG ("Unknown format");
+            GST_LOG ("Query using unknown format %d", *format);
             res = FALSE;
           }
           break;
@@ -520,7 +586,7 @@ dvdreadsrc_srcpad_query (GstPad * pad, GstQueryType type,
     case GST_QUERY_POSITION:
       switch (*format) {
         case GST_FORMAT_BYTES:
-          *value = priv->cur_pack * DVD_VIDEO_LB_LEN;
+          *value = (gint64) (priv->cur_pack) * DVD_VIDEO_LB_LEN;
           break;
         default:
           if (*format == sector_format) {
@@ -532,7 +598,7 @@ dvdreadsrc_srcpad_query (GstPad * pad, GstQueryType type,
           } else if (*format == angle_format) {
             *value = priv->angle;
           } else {
-            GST_LOG ("Unknown format");
+            GST_LOG ("Query using unknown format %d", *format);
             res = FALSE;
           }
           break;
@@ -608,7 +674,10 @@ _open (DVDReadSrcPrivate * priv, const gchar * location)
 static int
 _seek_title (DVDReadSrcPrivate * priv, int title, int angle)
 {
-  GHashTable *languagelist = NULL;
+  GstEvent *e;
+  GstStructure *s;
+  gint i;
+  gchar lang_code[3] = { '\0', '\0', '\0' }, *t;
 
     /**
      * Make sure our title number is valid.
@@ -670,18 +739,54 @@ _seek_title (DVDReadSrcPrivate * priv, int title, int angle)
     return -1;
   }
 
-  /* Get stream labels for all audio and subtitle streams */
-  languagelist = dvdreadsrc_init_languagelist ();
-
-  dvdreadsrc_get_audio_stream_labels (priv->vts_file, languagelist);
-  dvdreadsrc_get_subtitle_stream_labels (priv->vts_file, languagelist);
-
-  g_hash_table_destroy (languagelist);
-
   GST_LOG ("Opened title %d, angle %d", title, angle);
   priv->title = title;
   priv->angle = angle;
 
+  /* build event */
+  if (priv->title_lang_event_pending)
+    gst_event_unref (priv->title_lang_event_pending);
+  e = priv->title_lang_event_pending = gst_event_new (GST_EVENT_ANY);
+  s = e->event_data.structure.structure =
+      gst_structure_new ("application/x-gst-event",
+      "event", G_TYPE_STRING, "dvd-lang-codes", NULL);
+
+  /* audio */
+  for (i = 0; i < priv->vts_file->vtsi_mat->nr_of_vts_audio_streams; i++) {
+    const audio_attr_t *a = &priv->vts_file->vtsi_mat->vts_audio_attr[i];
+
+    t = g_strdup_printf ("audio-%d-format", i);
+    gst_structure_set (s, t, G_TYPE_INT, (int) a->audio_format, NULL);
+    g_free (t);
+
+    GST_DEBUG ("Audio stream %d is format %d", i, (int) a->audio_format);
+
+    if (a->lang_type) {
+      t = g_strdup_printf ("audio-%d-language", i);
+      lang_code[0] = (a->lang_code >> 8) & 0xff;
+      lang_code[1] = a->lang_code & 0xff;
+      gst_structure_set (s, t, G_TYPE_STRING, lang_code, NULL);
+      g_free (t);
+
+      GST_DEBUG ("Audio stream %d is language %s", i, lang_code);
+    }
+  }
+
+  /* subtitle */
+  for (i = 0; i < priv->vts_file->vtsi_mat->nr_of_vts_subp_streams; i++) {
+    const subp_attr_t *u = &priv->vts_file->vtsi_mat->vts_subp_attr[i];
+
+    if (u->type) {
+      t = g_strdup_printf ("subtitle-%d-language", i);
+      lang_code[0] = (u->lang_code >> 8) & 0xff;
+      lang_code[1] = u->lang_code & 0xff;
+      gst_structure_set (s, t, G_TYPE_STRING, lang_code, NULL);
+      g_free (t);
+    }
+
+    GST_DEBUG ("Subtitle stream %d is language %s", i, lang_code);
+  }
+
   return 0;
 }
 
@@ -707,6 +812,7 @@ _seek_chapter (DVDReadSrcPrivate * priv, int chapter)
      */
   priv->pgc_id = priv->vts_ptt_srpt->title[priv->ttn - 1].ptt[chapter].pgcn;
   priv->pgn = priv->vts_ptt_srpt->title[priv->ttn - 1].ptt[chapter].pgn;
+
   priv->cur_pgc = priv->vts_file->vts_pgcit->pgci_srp[priv->pgc_id - 1].pgc;
   priv->start_cell = priv->cur_pgc->program_map[priv->pgn - 1] - 1;
 
@@ -747,6 +853,13 @@ _seek_chapter (DVDReadSrcPrivate * priv, int chapter)
   priv->next_cell = priv->start_cell;
 
   priv->chapter = chapter;
+
+  if (priv->pending_clut_event)
+    gst_event_unref (priv->pending_clut_event);
+
+  priv->pending_clut_event =
+      dvdnavsrc_make_clut_change_event (priv, priv->cur_pgc->palette);
+
   return 0;
 }
 
@@ -1020,17 +1133,34 @@ dvdreadsrc_get (GstPad * pad)
             GST_FORMAT_UNDEFINED));
   }
 
+  if (priv->new_seek) {
+    _seek_title (priv, priv->title, priv->angle);
+    _seek_chapter (priv, priv->chapter);
+
+    priv->new_seek = FALSE;
+    priv->change_cell = TRUE;
+  }
+
+  if (priv->title_lang_event_pending) {
+    GstEvent *e = priv->title_lang_event_pending;
+
+    priv->title_lang_event_pending = NULL;
+    return GST_DATA (e);
+  }
+
+  if (priv->pending_clut_event) {
+    GstEvent *e = priv->pending_clut_event;
+
+    priv->pending_clut_event = NULL;
+    return GST_DATA (e);
+  }
+
   /* create the buffer */
   /* FIXME: should eventually use a bufferpool for this */
   buf = gst_buffer_new_and_alloc (1024 * DVD_VIDEO_LB_LEN);
 
-  if (priv->new_seek) {
-    _seek_title (priv, priv->title, priv->angle);
-    _seek_chapter (priv, priv->chapter);
-  }
-
   /* read it in from the file */
-  while ((res = _read (priv, priv->angle, priv->new_seek, buf)) == -3);
+  while ((res = _read (priv, priv->angle, priv->change_cell, buf)) == -3);
   switch (res) {
     case -1:
       GST_ELEMENT_ERROR (dvdreadsrc, RESOURCE, READ, (NULL), (NULL));
@@ -1046,7 +1176,7 @@ dvdreadsrc_get (GstPad * pad)
       g_assert_not_reached ();
   }
 
-  priv->new_seek = FALSE;
+  priv->change_cell = FALSE;
 
   return GST_DATA (buf);
 }
@@ -1100,11 +1230,20 @@ dvdreadsrc_change_state (GstElement * element)
     case GST_STATE_PAUSED_TO_READY:
       dvdreadsrc->priv->new_cell = TRUE;
       dvdreadsrc->priv->new_seek = TRUE;
+      dvdreadsrc->priv->change_cell = FALSE;
       dvdreadsrc->priv->chapter = 0;
       dvdreadsrc->priv->title = 0;
       dvdreadsrc->priv->flush_pend = FALSE;
       dvdreadsrc->priv->seek_pend = FALSE;
       dvdreadsrc->priv->seek_pend_fmt = GST_FORMAT_UNDEFINED;
+      if (dvdreadsrc->priv->title_lang_event_pending) {
+        gst_event_unref (dvdreadsrc->priv->title_lang_event_pending);
+        dvdreadsrc->priv->title_lang_event_pending = NULL;
+      }
+      if (dvdreadsrc->priv->pending_clut_event) {
+        gst_event_unref (dvdreadsrc->priv->pending_clut_event);
+        dvdreadsrc->priv->pending_clut_event = NULL;
+      }
       break;
     case GST_STATE_READY_TO_NULL:
       dvdreadsrc_close_file (DVDREADSRC (element));
@@ -1229,7 +1368,7 @@ dvdreadsrc_uri_handler_init (gpointer g_iface, gpointer iface_data)
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
-  if (!gst_element_register (plugin, "dvdreadsrc", GST_RANK_NONE,
+  if (!gst_element_register (plugin, "dvdreadsrc", GST_RANK_SECONDARY,
           GST_TYPE_DVDREADSRC))
     return FALSE;
 
