@@ -21,6 +21,8 @@
 #include "config.h"
 #endif
 
+#include <string.h>
+
 #include "gstfaac.h"
 
 static GstStaticPadTemplate src_template = GST_STATIC_PAD_TEMPLATE ("src",
@@ -34,10 +36,28 @@ static GstStaticPadTemplate src_template = GST_STATIC_PAD_TEMPLATE ("src",
 static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS ("audio/x-raw-int, " "endianness = (int) BYTE_ORDER, " "signed = (boolean) TRUE, " "width = (int) 16, " "depth = (int) 16, " "rate = (int) [ 8000, 96000 ], " "channels = (int) [ 1, 6]; " "audio/x-raw-int, " "endianness = (int) BYTE_ORDER, " "signed = (boolean) TRUE, " "width = (int) 32, " "depth = (int) 24, " "rate = (int) [ 8000, 96000], " "channels = (int) [ 1, 6]; " "audio/x-raw-float, " "endianness = (int) BYTE_ORDER, " "depth = (int) 32, "    /* sizeof (gfloat) */
-        "rate = (int) [ 8000, 96000], " "channels = (int) [ 1, 6]")
+    GST_STATIC_CAPS ("audio/x-raw-int, "
+        "endianness = (int) BYTE_ORDER, "
+        "signed = (boolean) TRUE, "
+        "width = (int) 16, "
+        "depth = (int) 16, "
+        "rate = (int) [ 8000, 96000 ], " "channels = (int) [ 1, 6]")
     );
 
+#define UNUSED \
+    "audio/x-raw-int, " \
+    "endianness = (int) BYTE_ORDER, " \
+    "signed = (boolean) TRUE, " \
+    "width = (int) 32, " \
+    "depth = (int) 24, " \
+    "rate = (int) [ 8000, 96000], " \
+    "channels = (int) [ 1, 6]; " \
+    "audio/x-raw-float, " \
+    "endianness = (int) BYTE_ORDER, " \
+    "depth = (int) 32, "    /* sizeof (gfloat) */ \
+    "rate = (int) [ 8000, 96000], " \
+    "channels = (int) [ 1, 6]" \
+
 enum
 {
   ARG_0,
@@ -61,8 +81,7 @@ static void gst_faac_get_property (GObject * object,
 
 static GstPadLinkReturn
 gst_faac_sinkconnect (GstPad * pad, const GstCaps * caps);
-static GstPadLinkReturn
-gst_faac_srcconnect (GstPad * pad, const GstCaps * caps);
+static GstPadLinkReturn gst_faac_srcconnect (GstPad * pad);
 static void gst_faac_chain (GstPad * pad, GstData * data);
 static GstElementStateReturn gst_faac_change_state (GstElement * element);
 
@@ -221,23 +240,22 @@ gst_faac_init (GstFaac * faac)
   faac->samplerate = -1;
   faac->channels = -1;
   faac->cache = NULL;
-  faac->cache_time = GST_CLOCK_TIME_NONE;
-  faac->cache_duration = 0;
+  faac->out_time = 0;
 
   GST_FLAG_SET (faac, GST_ELEMENT_EVENT_AWARE);
 
   faac->sinkpad =
       gst_pad_new_from_template (gst_static_pad_template_get (&sink_template),
       "sink");
-  gst_element_add_pad (GST_ELEMENT (faac), faac->sinkpad);
   gst_pad_set_chain_function (faac->sinkpad, gst_faac_chain);
   gst_pad_set_link_function (faac->sinkpad, gst_faac_sinkconnect);
+  gst_element_add_pad (GST_ELEMENT (faac), faac->sinkpad);
 
   faac->srcpad =
       gst_pad_new_from_template (gst_static_pad_template_get (&src_template),
       "src");
+  gst_pad_use_explicit_caps (faac->srcpad);
   gst_element_add_pad (GST_ELEMENT (faac), faac->srcpad);
-  gst_pad_set_link_function (faac->srcpad, gst_faac_srcconnect);
 
   /* default properties */
   faac->bitrate = 1024 * 128;
@@ -307,39 +325,33 @@ gst_faac_sinkconnect (GstPad * pad, const GstCaps * caps)
 
   /* if the other side was already set-up, redo that */
   if (GST_PAD_CAPS (faac->srcpad))
-    return gst_faac_srcconnect (faac->srcpad,
-        gst_pad_get_allowed_caps (faac->srcpad));
+    return gst_faac_srcconnect (faac->srcpad);
 
   /* else, that'll be done later */
   return GST_PAD_LINK_OK;
 }
 
 static GstPadLinkReturn
-gst_faac_srcconnect (GstPad * pad, const GstCaps * caps)
+gst_faac_srcconnect (GstPad * pad)
 {
   GstFaac *faac = GST_FAAC (gst_pad_get_parent (pad));
+  GstCaps *caps;
   gint n;
 
   if (!faac->handle || (faac->samplerate == -1 || faac->channels == -1)) {
     return GST_PAD_LINK_DELAYED;
   }
 
-  /* we do samplerate/channels ourselves */
-  for (n = 0; n < gst_caps_get_size (caps); n++) {
-    GstStructure *structure = gst_caps_get_structure (caps, n);
-
-    gst_structure_remove_field (structure, "rate");
-    gst_structure_remove_field (structure, "channels");
-  }
-
   /* go through list */
-  caps = gst_caps_normalize (caps);
+  caps = gst_pad_get_allowed_caps (pad);
   for (n = 0; n < gst_caps_get_size (caps); n++) {
     GstStructure *structure = gst_caps_get_structure (caps, n);
     faacEncConfiguration *conf;
-    gint mpegversion = 0;
+    gint mpegversion = 4;
     GstCaps *newcaps;
     GstPadLinkReturn ret;
+    guint8 *c_data;
+    gulong c_size;
 
     gst_structure_get_int (structure, "mpegversion", &mpegversion);
 
@@ -363,19 +375,26 @@ gst_faac_srcconnect (GstPad * pad, const GstCaps * caps)
         "mpegversion", G_TYPE_INT, mpegversion,
         "channels", G_TYPE_INT, faac->channels,
         "rate", G_TYPE_INT, faac->samplerate, NULL);
-    ret = gst_pad_try_set_caps (faac->srcpad, newcaps);
+    if ((ret =
+            faacEncGetDecoderSpecificInfo (faac->handle, &c_data,
+                &c_size)) == 0) {
+      GstBuffer *data = gst_buffer_new_and_alloc (c_size);
 
-    switch (ret) {
-      case GST_PAD_LINK_OK:
-      case GST_PAD_LINK_DONE:
-        return GST_PAD_LINK_DONE;
-      case GST_PAD_LINK_DELAYED:
-        return GST_PAD_LINK_DELAYED;
-      default:
-        break;
+      memcpy (GST_BUFFER_DATA (data), c_data, c_size);
+      gst_caps_set_simple (newcaps, "codec_data", GST_TYPE_BUFFER, data, NULL);
+      gst_buffer_unref (data);
     }
+    if (gst_pad_set_explicit_caps (faac->srcpad, newcaps)) {
+      gst_caps_free (caps);
+      gst_caps_free (newcaps);
+      return GST_PAD_LINK_OK;
+    }
+
+    gst_caps_free (newcaps);
   }
 
+  gst_caps_free (caps);
+
   return GST_PAD_LINK_REFUSED;
 }
 
@@ -404,8 +423,10 @@ gst_faac_chain (GstPad * pad, GstData * data)
 
           if (ret_size > 0) {
             GST_BUFFER_SIZE (outbuf) = ret_size;
-            GST_BUFFER_TIMESTAMP (outbuf) = 0;
-            GST_BUFFER_DURATION (outbuf) = 0;
+            GST_BUFFER_TIMESTAMP (outbuf) = faac->out_time;
+            GST_BUFFER_DURATION (outbuf) = GST_SECOND * faac->samples /
+                (faac->channels * faac->samplerate);
+            faac->out_time += GST_BUFFER_DURATION (outbuf);
             gst_pad_push (faac->srcpad, GST_DATA (outbuf));
           } else {
             break;
@@ -431,8 +452,7 @@ gst_faac_chain (GstPad * pad, GstData * data)
   }
 
   if (!GST_PAD_CAPS (faac->srcpad)) {
-    if (gst_faac_srcconnect (faac->srcpad,
-            gst_pad_get_allowed_caps (faac->srcpad)) <= 0) {
+    if (gst_faac_srcconnect (faac->srcpad) <= 0) {
       GST_ELEMENT_ERROR (faac, CORE, NEGOTIATION, (NULL),
           ("failed to negotiate MPEG/AAC format with next element"));
       gst_buffer_unref (inbuf);
@@ -511,30 +531,11 @@ gst_faac_chain (GstPad * pad, GstData * data)
 
     if (ret_size > 0) {
       GST_BUFFER_SIZE (outbuf) = ret_size;
-      if (faac->cache_time != GST_CLOCK_TIME_NONE) {
-        GST_BUFFER_TIMESTAMP (outbuf) = faac->cache_time;
-        faac->cache_time = GST_CLOCK_TIME_NONE;
-      } else
-        GST_BUFFER_TIMESTAMP (outbuf) = GST_BUFFER_TIMESTAMP (subbuf);
-      GST_BUFFER_DURATION (outbuf) = GST_BUFFER_DURATION (subbuf);
-      if (faac->cache_duration) {
-        GST_BUFFER_DURATION (outbuf) += faac->cache_duration;
-        faac->cache_duration = 0;
-      }
+      GST_BUFFER_TIMESTAMP (outbuf) = faac->out_time;
+      GST_BUFFER_DURATION (outbuf) = GST_SECOND * faac->samples /
+          (faac->channels * faac->samplerate);
+      faac->out_time += GST_BUFFER_DURATION (outbuf);
       gst_pad_push (faac->srcpad, GST_DATA (outbuf));
-    } else {
-      /* FIXME: what I'm doing here isn't fully correct, but there
-       * really isn't a better way yet.
-       * Problem is that libfaac caches buffers (for encoding
-       * purposes), so the timestamp of the outgoing buffer isn't
-       * the same as the timestamp of the data that I pushed in.
-       * However, I don't know the delay between those two so I
-       * cannot really say aything about it. This is a bad guess. */
-
-      gst_buffer_unref (outbuf);
-      if (faac->cache_time != GST_CLOCK_TIME_NONE)
-        faac->cache_time = GST_BUFFER_TIMESTAMP (subbuf);
-      faac->cache_duration += GST_BUFFER_DURATION (subbuf);
     }
 
     in_size -= frame_size;
@@ -619,8 +620,7 @@ gst_faac_change_state (GstElement * element)
         gst_buffer_unref (faac->cache);
         faac->cache = NULL;
       }
-      faac->cache_time = GST_CLOCK_TIME_NONE;
-      faac->cache_duration = 0;
+      faac->out_time = 0;
       faac->samplerate = -1;
       faac->channels = -1;
       break;
