@@ -82,7 +82,7 @@ static GstPadTemplate *gst_vorbisdec_src_template, *gst_vorbisdec_sink_template;
 /* elementfactory information */
 GstElementDetails ivorbisfile_details = {
   "Ogg Vorbis decoder",
-  "Codec/Audio/Decoder",
+  "Codec/Decoder/Audio",
   "Decodes OGG Vorbis audio using the Tremor vorbisfile API",
   "Monty <monty@xiph.org>\n"
       "Wim Taymans <wim.taymans@chello.be>\n"
@@ -354,7 +354,7 @@ gst_ivorbisfile_seek (void *datasource, int64_t offset, int whence)
     return -1;
   }
 
-  GST_DEBUG ("seek %lld %d", offset, whence);
+  GST_DEBUG ("seek %" G_GINT64_FORMAT " %d", offset, whence);
 
   if (whence == SEEK_SET) {
     method = GST_SEEK_METHOD_SET;
@@ -858,6 +858,7 @@ gst_ivorbisfile_get_query_types (GstPad * pad)
     GST_QUERY_POSITION,
     0
   };
+
   return types;
 }
 
@@ -956,6 +957,7 @@ gst_ivorbisfile_get_event_masks (GstPad * pad)
     {GST_EVENT_SEEK, GST_SEEK_METHOD_SET | GST_SEEK_FLAG_ACCURATE},
     {0,}
   };
+
   return masks;
 }
 
