@@ -234,40 +234,46 @@ gst_cacasink_sinkconnect (GstPad * pad, const GstCaps * caps)
 {
   GstCACASink *cacasink;
   GstStructure *structure;
+  gint width, height, bpp, red_mask, green_mask, blue_mask;
 
   cacasink = GST_CACASINK (gst_pad_get_parent (pad));
 
   /* We cannot use library functions if the sink is not open */
   if (!GST_FLAG_IS_SET (GST_ELEMENT (cacasink), GST_CACASINK_OPEN))
-     return GST_PAD_LINK_DELAYED;
+    return GST_PAD_LINK_DELAYED;
   /*if (!GST_CAPS_IS_FIXED (caps))
      return GST_PAD_LINK_DELAYED; */
 
   structure = gst_caps_get_structure (caps, 0);
-  gst_structure_get_int (structure, "width", &(GST_VIDEOSINK_WIDTH (cacasink)));
-  gst_structure_get_int (structure, "height",
-      &(GST_VIDEOSINK_HEIGHT (cacasink)));
-  gst_structure_get_int (structure, "bpp", &cacasink->bpp);
-  gst_structure_get_int (structure, "red_mask", &cacasink->red_mask);
-  gst_structure_get_int (structure, "green_mask", &cacasink->green_mask);
-  gst_structure_get_int (structure, "blue_mask", &cacasink->blue_mask);
+  if (!gst_structure_get_int (structure, "width", &width)
+      || !gst_structure_get_int (structure, "height", &height)
+      || !gst_structure_get_int (structure, "bpp", &bpp)
+      || !gst_structure_get_int (structure, "red_mask", &red_mask)
+      || !gst_structure_get_int (structure, "green_mask", &green_mask)
+      || !gst_structure_get_int (structure, "blue_mask", &blue_mask)) {
+    return GST_PAD_LINK_REFUSED;
+  }
+
+  GST_VIDEOSINK_WIDTH (cacasink) = width;
+  GST_VIDEOSINK_HEIGHT (cacasink) = height;
+  cacasink->bpp = bpp;
 
   if (cacasink->bpp == 24) {
-    cacasink->red_mask = GUINT32_FROM_BE (cacasink->red_mask) >> 8;
-    cacasink->green_mask = GUINT32_FROM_BE (cacasink->green_mask) >> 8;
-    cacasink->blue_mask = GUINT32_FROM_BE (cacasink->blue_mask) >> 8;
+    cacasink->red_mask = GUINT32_FROM_BE (red_mask) >> 8;
+    cacasink->green_mask = GUINT32_FROM_BE (green_mask) >> 8;
+    cacasink->blue_mask = GUINT32_FROM_BE (blue_mask) >> 8;
   }
 
   else if (cacasink->bpp == 32) {
-    cacasink->red_mask = GUINT32_FROM_BE (cacasink->red_mask);
-    cacasink->green_mask = GUINT32_FROM_BE (cacasink->green_mask);
-    cacasink->blue_mask = GUINT32_FROM_BE (cacasink->blue_mask);
+    cacasink->red_mask = GUINT32_FROM_BE (red_mask);
+    cacasink->green_mask = GUINT32_FROM_BE (green_mask);
+    cacasink->blue_mask = GUINT32_FROM_BE (blue_mask);
   }
 
   else if (cacasink->bpp == 16 || cacasink->bpp == 15) {
-    cacasink->red_mask = GUINT16_FROM_BE (cacasink->red_mask);
-    cacasink->green_mask = GUINT16_FROM_BE (cacasink->green_mask);
-    cacasink->blue_mask = GUINT16_FROM_BE (cacasink->blue_mask);
+    cacasink->red_mask = GUINT16_FROM_BE (red_mask);
+    cacasink->green_mask = GUINT16_FROM_BE (green_mask);
+    cacasink->blue_mask = GUINT16_FROM_BE (blue_mask);
   }
 
   if (cacasink->bitmap) {
@@ -313,8 +319,6 @@ gst_cacasink_chain (GstPad * pad, GstData * _data)
 {
   GstBuffer *buf = GST_BUFFER (_data);
   GstCACASink *cacasink;
-  GstClockTime time = GST_BUFFER_TIMESTAMP (buf);
-  gint64 jitter;
 
   g_return_if_fail (pad != NULL);
   g_return_if_fail (GST_IS_PAD (pad));
@@ -325,10 +329,10 @@ gst_cacasink_chain (GstPad * pad, GstData * _data)
   if (!GST_FLAG_IS_SET (GST_ELEMENT (cacasink), GST_CACASINK_OPEN))
     return;
 
-  if (cacasink->id && GST_CLOCK_TIME_IS_VALID (time)) {
-    GST_DEBUG ("videosink: clock %s wait: %" G_GUINT64_FORMAT " %u",
-        GST_OBJECT_NAME (GST_VIDEOSINK_CLOCK (cacasink)),
-        time, GST_BUFFER_SIZE (buf));
+  GST_DEBUG ("videosink: clock wait: %" G_GUINT64_FORMAT,
+      GST_BUFFER_TIMESTAMP (buf));
+
+  if (GST_VIDEOSINK_CLOCK (cacasink) && GST_BUFFER_TIMESTAMP_IS_VALID (buf)) {
     gst_element_wait (GST_ELEMENT (cacasink), GST_BUFFER_TIMESTAMP (buf));
   }
 
@@ -337,13 +341,6 @@ gst_cacasink_chain (GstPad * pad, GstData * _data)
       cacasink->screen_height - 1, cacasink->bitmap, GST_BUFFER_DATA (buf));
   caca_refresh ();
 
-  if (GST_VIDEOSINK_CLOCK (cacasink)) {
-    jitter = gst_clock_get_time (GST_VIDEOSINK_CLOCK (cacasink)) - time;
-
-    cacasink->correction = (cacasink->correction + jitter) >> 1;
-    cacasink->correction = 0;
-  }
-
   gst_buffer_unref (buf);
 }
 
