@@ -69,8 +69,8 @@ static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
         "format = (fourcc) I420, "
         "width = (int) [ 16, 4096 ], "
         "height = (int) [ 16, 4096 ], "
-        "framerate = (double) { 23.976024, 24.0, 25.0, 29.970030, 30.0, "
-        "  50.0, 59.940060, 60.0 }")
+        "framerate = (double) { 23.976023976, 24.0, 25.0, 29.97002997, 30.0, "
+        "  50.0, 59.94005994, 60.0 }")
     );
 
 static GstStaticPadTemplate src_template = GST_STATIC_PAD_TEMPLATE ("src",
@@ -81,8 +81,8 @@ static GstStaticPadTemplate src_template = GST_STATIC_PAD_TEMPLATE ("src",
         "systemstream = (boolean) FALSE, "
         "width = (int) [ 16, 4096 ], "
         "height = (int) [ 16, 4096 ], "
-        "framerate = (double) { 23.976024, 24.0, 25.0, 29.970030, 30.0, "
-        "  50.0, 59.940060, 60.0 }")
+        "framerate = (double) { 23.976023976, 24.0, 25.0, 29.97002997, 30.0, "
+        "  50.0, 59.94005994, 60.0 }")
     );
 
 #define MAX_FRAME_RATES  9
@@ -132,7 +132,7 @@ framerate_to_index (gfloat fps)
 static void gst_fameenc_class_init (GstFameEncClass * klass);
 static void gst_fameenc_base_init (GstFameEncClass * klass);
 static void gst_fameenc_init (GstFameEnc * fameenc);
-static void gst_fameenc_dispose (GObject * object);
+static void gst_fameenc_finalize (GObject * object);
 
 static void gst_fameenc_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
@@ -205,7 +205,7 @@ gst_fameenc_class_init (GstFameEncClass * klass)
 
   gobject_class->set_property = gst_fameenc_set_property;
   gobject_class->get_property = gst_fameenc_get_property;
-  gobject_class->dispose = gst_fameenc_dispose;
+  gobject_class->finalize = gst_fameenc_finalize;
 
   fame_object_name = g_quark_from_string ("GstFameObjectName");
 
@@ -262,15 +262,16 @@ gst_fameenc_class_init (GstFameEncClass * klass)
     if (array->len > 0) {
       GType type;
       GParamSpec *pspec;
+      gchar *str1, *str2;
 
-      type =
-          g_enum_register_static (g_strdup_printf ("GstFameEnc_%s",
-              current_type), (GEnumValue *) array->data);
+      str1 = g_strdup_printf ("GstFameEnc_%s", current_type);
+      type = g_enum_register_static (str1, (GEnumValue *) array->data);
+      g_free (str1);
 
-      pspec =
-          g_param_spec_enum (current_type, current_type,
-          g_strdup_printf ("The FAME \"%s\" object", current_type), type,
+      str2 = g_strdup_printf ("The FAME \"%s\" object", current_type);
+      pspec = g_param_spec_enum (current_type, current_type, str2, type,
           default_index, G_PARAM_READWRITE);
+      g_free (str2);
 
       g_param_spec_set_qdata (pspec, fame_object_name, (gpointer) current_type);
 
@@ -402,13 +403,13 @@ gst_fameenc_init (GstFameEnc * fameenc)
 }
 
 static void
-gst_fameenc_dispose (GObject * object)
+gst_fameenc_finalize (GObject * object)
 {
   GstFameEnc *fameenc = GST_FAMEENC (object);
 
-  G_OBJECT_CLASS (parent_class)->dispose (object);
-
   g_free (fameenc->buffer);
+
+  G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static void
