@@ -47,8 +47,6 @@ static void gst_pngdec_init (GstPngDec * pngdec);
 
 static void gst_pngdec_chain (GstPad * pad, GstData * _data);
 
-static GstCaps *gst_pngdec_src_getcaps (GstPad * pad);
-
 static GstElementClass *parent_class = NULL;
 
 
@@ -100,7 +98,7 @@ static GstStaticPadTemplate gst_pngdec_sink_pad_template =
 GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS ("video/x-png, "
+    GST_STATIC_CAPS ("image/png, "
         "width = (int) [ 16, 4096 ], "
         "height = (int) [ 16, 4096 ], " "framerate = (double) [ 0.0, MAX ]")
     );
@@ -129,7 +127,6 @@ gst_pngdec_class_init (GstPngDecClass * klass)
   parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
 }
 
-
 static GstPadLinkReturn
 gst_pngdec_sinklink (GstPad * pad, const GstCaps * caps)
 {
@@ -158,7 +155,7 @@ gst_pngdec_init (GstPngDec * pngdec)
   gst_pad_set_chain_function (pngdec->sinkpad, gst_pngdec_chain);
   gst_pad_set_link_function (pngdec->sinkpad, gst_pngdec_sinklink);
 
-  gst_pad_set_getcaps_function (pngdec->srcpad, gst_pngdec_src_getcaps);
+  gst_pad_use_explicit_caps (pngdec->srcpad);
 
   pngdec->png = NULL;
   pngdec->info = NULL;
@@ -169,65 +166,6 @@ gst_pngdec_init (GstPngDec * pngdec)
   pngdec->fps = -1;
 }
 
-static GstCaps *
-gst_pngdec_src_getcaps (GstPad * pad)
-{
-  GstPngDec *pngdec;
-  GstCaps *caps;
-  gint i;
-  GstPadTemplate *templ;
-  GstCaps *inter;
-
-  pngdec = GST_PNGDEC (gst_pad_get_parent (pad));
-  templ = gst_static_pad_template_get (&gst_pngdec_src_pad_template);
-  caps = gst_caps_copy (gst_pad_template_get_caps (templ));
-
-  if (pngdec->color_type != -1) {
-    GstCaps *to_inter = NULL;
-
-    switch (pngdec->color_type) {
-      case PNG_COLOR_TYPE_RGB:
-        to_inter = gst_caps_new_simple ("video/x-raw-rgb",
-            "bpp", G_TYPE_INT, 24, NULL);
-        break;
-      case PNG_COLOR_TYPE_RGB_ALPHA:
-        to_inter = gst_caps_new_simple ("video/x-raw-rgb",
-            "bpp", G_TYPE_INT, 32, NULL);
-        break;
-      case PNG_COLOR_TYPE_GRAY:
-      case PNG_COLOR_TYPE_PALETTE:
-      case PNG_COLOR_TYPE_GRAY_ALPHA:
-      default:
-        GST_ELEMENT_ERROR (pngdec, STREAM, NOT_IMPLEMENTED, (NULL),
-            ("pngdec does not support grayscale or paletted data yet"));
-        break;
-    }
-    inter = gst_caps_intersect (caps, to_inter);
-    gst_caps_free (caps);
-    gst_caps_free (to_inter);
-    caps = inter;
-  }
-
-  for (i = 0; i < gst_caps_get_size (caps); i++) {
-    GstStructure *structure = gst_caps_get_structure (caps, i);
-
-    if (pngdec->width != -1) {
-      gst_structure_set (structure, "width", G_TYPE_INT, pngdec->width, NULL);
-    }
-
-    if (pngdec->height != -1) {
-      gst_structure_set (structure, "height", G_TYPE_INT, pngdec->height, NULL);
-    }
-
-    if (pngdec->fps != -1) {
-      gst_structure_set (structure,
-          "framerate", G_TYPE_DOUBLE, pngdec->fps, NULL);
-    }
-  }
-
-  return caps;
-}
-
 static void
 user_read_data (png_structp png_ptr, png_bytep data, png_size_t length)
 {
@@ -244,6 +182,8 @@ user_read_data (png_structp png_ptr, png_bytep data, png_size_t length)
   dec->offset += length;
 }
 
+#define ROUND_UP_4(x) (((x) + 3) & ~3)
+
 static void
 gst_pngdec_chain (GstPad * pad, GstData * _data)
 {
@@ -269,30 +209,38 @@ gst_pngdec_chain (GstPad * pad, GstData * _data)
   pngdec->png = png_create_read_struct (PNG_LIBPNG_VER_STRING,
       (png_voidp) NULL, user_error_fn, user_warning_fn);
 
-  /* FIXME: better error handling */
   if (pngdec->png == NULL) {
-    g_warning ("Failed to initialize png structure");
+    gst_buffer_unref (buf);
+    GST_ELEMENT_ERROR (pngdec, LIBRARY, INIT, (NULL),
+        ("Failed to initialize png structure"));
     return;
   }
 
   pngdec->info = png_create_info_struct (pngdec->png);
   if (pngdec->info == NULL) {
-    g_warning ("Failed to initialize info structure");
+    gst_buffer_unref (buf);
     png_destroy_read_struct (&(pngdec->png), (png_infopp) NULL,
         (png_infopp) NULL);
+    GST_ELEMENT_ERROR (pngdec, LIBRARY, INIT, (NULL),
+        ("Failed to initialize info structure"));
     return;
   }
 
   pngdec->endinfo = png_create_info_struct (pngdec->png);
   if (pngdec->endinfo == NULL) {
-    g_warning ("Failed to initialize endinfo structure");
+    gst_buffer_unref (buf);
+    png_destroy_read_struct (&pngdec->png, &pngdec->info, (png_infopp) NULL);
+    GST_ELEMENT_ERROR (pngdec, LIBRARY, INIT, (NULL),
+        ("Failed to initialize endinfo structure"));
     return;
   }
 
   /* non-0 return is from a longjmp inside of libpng */
   if (setjmp (pngdec->png->jmpbuf) != 0) {
-    GST_DEBUG ("returning from longjmp");
+    gst_buffer_unref (buf);
     png_destroy_read_struct (&pngdec->png, &pngdec->info, &pngdec->endinfo);
+    GST_ELEMENT_ERROR (pngdec, LIBRARY, FAILED, (NULL),
+        ("returning from longjmp"));
     return;
   }
 
@@ -302,16 +250,55 @@ gst_pngdec_chain (GstPad * pad, GstData * _data)
   png_get_IHDR (pngdec->png, pngdec->info, &width, &height,
       &depth, &color, NULL, NULL, NULL);
 
-  GST_LOG ("color type: %d\n", pngdec->info->color_type);
+  if (pngdec->info->bit_depth != 8) {
+    gst_buffer_unref (buf);
+    png_destroy_read_struct (&pngdec->png, &pngdec->info, &pngdec->endinfo);
+    GST_ELEMENT_ERROR (pngdec, STREAM, NOT_IMPLEMENTED, (NULL),
+        ("pngdec only supports 8 bit images for now"));
+    return;
+  }
 
   if (pngdec->width != width ||
       pngdec->height != height ||
       pngdec->color_type != pngdec->info->color_type) {
+    GstCaps *caps, *templ, *res;
+    gboolean ret;
+    gint bpp;
+
     pngdec->width = width;
     pngdec->height = height;
     pngdec->color_type = pngdec->info->color_type;
 
-    if (GST_PAD_LINK_FAILED (gst_pad_renegotiate (pngdec->srcpad))) {
+    templ = gst_caps_copy (gst_pad_template_get_caps
+        (gst_static_pad_template_get (&gst_pngdec_src_pad_template)));
+
+    switch (pngdec->color_type) {
+      case PNG_COLOR_TYPE_RGB:
+        bpp = 24;
+        break;
+      case PNG_COLOR_TYPE_RGB_ALPHA:
+        bpp = 32;
+        break;
+      default:
+        GST_ELEMENT_ERROR (pngdec, STREAM, NOT_IMPLEMENTED, (NULL),
+            ("pngdec does not support grayscale or paletted data yet"));
+        return;
+    }
+
+    caps = gst_caps_new_simple ("video/x-raw-rgb",
+        "width", G_TYPE_INT, width,
+        "height", G_TYPE_INT, height,
+        "bpp", G_TYPE_INT, bpp, "framerate", G_TYPE_DOUBLE, pngdec->fps, NULL);
+
+    res = gst_caps_intersect (templ, caps);
+    gst_caps_free (caps);
+    gst_caps_free (templ);
+    ret = gst_pad_set_explicit_caps (pngdec->srcpad, res);
+    gst_caps_free (res);
+
+    if (!ret) {
+      gst_buffer_unref (buf);
+      png_destroy_read_struct (&pngdec->png, &pngdec->info, &pngdec->endinfo);
       GST_ELEMENT_ERROR (pngdec, CORE, NEGOTIATION, (NULL), (NULL));
       return;
     }
@@ -319,14 +306,13 @@ gst_pngdec_chain (GstPad * pad, GstData * _data)
 
   rows = (png_bytep *) g_malloc (sizeof (png_bytep) * height);
 
-  out =
-      gst_pad_alloc_buffer (pngdec->srcpad, GST_BUFFER_OFFSET_NONE,
-      width * height * 4);
+  out = gst_pad_alloc_buffer (pngdec->srcpad, GST_BUFFER_OFFSET_NONE,
+      height * ROUND_UP_4 (pngdec->info->rowbytes));
 
   inp = GST_BUFFER_DATA (out);
   for (i = 0; i < height; i++) {
     rows[i] = inp;
-    inp += width * 4;
+    inp += ROUND_UP_4 (pngdec->info->rowbytes);
   }
 
   png_read_image (pngdec->png, rows);
