@@ -26,6 +26,12 @@
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
+  if (!gst_library_load ("gstbytestream"))
+    return FALSE;
+
+  if (!gst_library_load ("gsttags"))
+    return FALSE;
+
   if (!gst_element_register (plugin, "speexenc", GST_RANK_NONE,
           GST_TYPE_SPEEXENC))
     return FALSE;
