@@ -196,7 +196,7 @@ gst_speexenc_class_init (GstSpeexEncClass * klass)
           0.0, 10.0, DEFAULT_QUALITY, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_BITRATE,
       g_param_spec_int ("bitrate", "Encoding Bit-rate",
-          "Specify an encoding bit-rate (in bps). ",
+          "Specify an encoding bit-rate (in bps). (0 = automatic)",
           0, G_MAXINT, DEFAULT_BITRATE, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_VBR,
       g_param_spec_boolean ("vbr", "VBR",
@@ -883,13 +883,13 @@ gst_speexenc_chain (GstPad * pad, GstData * _data)
       gst_speexenc_set_metadata (speexenc);
 
       /* create header buffer */
-      data = speex_header_to_packet (&speexenc->header, &data_len);
+      data = (guchar *) speex_header_to_packet (&speexenc->header, &data_len);
       buf1 = gst_speexenc_buffer_from_data (speexenc, data, data_len, 0);
 
       /* create comment buffer */
       buf2 =
-          gst_speexenc_buffer_from_data (speexenc, speexenc->comments,
-          speexenc->comment_len, 0);
+          gst_speexenc_buffer_from_data (speexenc,
+          (guchar *) speexenc->comments, speexenc->comment_len, 0);
 
       /* mark and put on caps */
       caps = gst_pad_get_caps (speexenc->srcpad);
@@ -947,11 +947,15 @@ gst_speexenc_chain (GstPad * pad, GstData * _data)
             gst_pad_alloc_buffer (speexenc->srcpad, GST_BUFFER_OFFSET_NONE,
             outsize);
         written =
-            speex_bits_write (&speexenc->bits, GST_BUFFER_DATA (outbuf),
-            outsize);
+            speex_bits_write (&speexenc->bits,
+            (gchar *) GST_BUFFER_DATA (outbuf), outsize);
         g_assert (written == outsize);
         speex_bits_reset (&speexenc->bits);
 
+        GST_BUFFER_TIMESTAMP (outbuf) =
+            (speexenc->frameno * frame_size -
+            speexenc->lookahead) * GST_SECOND / speexenc->rate;
+        GST_BUFFER_DURATION (outbuf) = frame_size * GST_SECOND / speexenc->rate;
         GST_BUFFER_OFFSET (outbuf) = speexenc->bytes_out;
         GST_BUFFER_OFFSET_END (outbuf) =
             speexenc->frameno * frame_size - speexenc->lookahead;
