@@ -21,7 +21,6 @@
 #define __GST_XVIDDEC_H__
 
 #include <gst/gst.h>
-#include <xvid.h>
 #include "gstxvid.h"
 
 #ifdef __cplusplus
@@ -53,9 +52,9 @@ struct _GstXvidDec {
   void *handle;
 
   /* video (output) settings */
-  int csp, bpp;
-  int width, height;
-  float fps;
+  gint csp, bpp, stride;
+  gint width, height;
+  double fps;
 };
 
 struct _GstXvidDecClass {
