@@ -276,6 +276,7 @@ gst_adder_request_new_pad (GstElement * element, GstPadTemplate * templ,
   name = g_strdup_printf ("sink%d", adder->numsinkpads);
   input->sinkpad = gst_pad_new_from_template (templ, name);
   input->bytestream = gst_bytestream_new (input->sinkpad);
+  GST_DEBUG_OBJECT (adder, "created new bytestream %p", input->bytestream);
   input->eos = FALSE;
 
   gst_element_add_pad (GST_ELEMENT (adder), input->sinkpad);
@@ -284,6 +285,8 @@ gst_adder_request_new_pad (GstElement * element, GstPadTemplate * templ,
 
   /* add the input_channel to the list of input channels */
 
+  GST_DEBUG_OBJECT (adder, "adding input channel %p with sinkpad %s:%s",
+      input, GST_DEBUG_PAD_NAME (input->sinkpad));
   adder->input_channels = g_slist_append (adder->input_channels, input);
   adder->numsinkpads++;
 
@@ -295,6 +298,7 @@ static void
 gst_adder_release_pad (GstElement * element, GstPad * pad)
 {
   GstAdder *adder;
+  gboolean found = FALSE;
   GSList *inputs;
 
   if (!GST_IS_ADDER (element))
@@ -302,22 +306,34 @@ gst_adder_release_pad (GstElement * element, GstPad * pad)
 
   adder = GST_ADDER (element);
 
+  GST_DEBUG_OBJECT (adder, "asked to release pad %s:%s",
+      GST_DEBUG_PAD_NAME (pad));
+
   inputs = adder->input_channels;
 
   while (inputs) {
     GstAdderInputChannel *input;
 
     input = (GstAdderInputChannel *) inputs->data;
+    GST_DEBUG_OBJECT (adder, "looking at input channel %p with sinkpad %s:%s",
+      input, GST_DEBUG_PAD_NAME (input->sinkpad));
     if (input->sinkpad == pad) {
-      GST_LOG_OBJECT (adder, "  found pad");
+      GST_DEBUG_OBJECT (adder, "found pad to release: %s:%s",
+          GST_DEBUG_PAD_NAME (pad));
+      GST_DEBUG_OBJECT (adder, "destroying bytestream %p", input->bytestream);
       gst_bytestream_destroy (input->bytestream);
       adder->input_channels = g_slist_remove (adder->input_channels, input);
       g_free (input);
       gst_element_remove_pad (element, pad);
+      found = TRUE;
       break;
     }
     inputs = inputs->next;
   }
+  if (!found) {
+    g_warning ("the pad %s:%s given as an argument is not adder's !",
+        GST_DEBUG_PAD_NAME (pad));
+  }
 }
 
 static void
@@ -449,7 +465,7 @@ gst_adder_loop (GstElement * element)
           case GST_EVENT_EOS:
             /* if we get an EOS event from one of our sink pads, we assume that
                pad's finished handling data. just skip this pad. */
-            GST_DEBUG ("    got an EOS event");
+            GST_DEBUG ("    got an EOSish event");
             gst_event_unref (event);
             input->eos = TRUE;
             valid_inputs--;
@@ -559,14 +575,17 @@ gst_adder_loop (GstElement * element)
   }
 
   if (valid_inputs > 0 || adder->eos == FALSE) {
+    gint64 delta;
+    gint64 duration;
+
+    delta = GST_BUFFER_SIZE (buf_out) * 8 / (adder->width * adder->channels);
+    duration = delta * GST_SECOND / adder->rate;
+
     GST_BUFFER_TIMESTAMP (buf_out) = adder->timestamp;
-    if (adder->format == GST_ADDER_FORMAT_FLOAT)
-      adder->offset +=
-          GST_BUFFER_SIZE (buf_out) / adder->width / adder->channels;
-    else
-      adder->offset +=
-          GST_BUFFER_SIZE (buf_out) * 8 / adder->width / adder->channels;
-    adder->timestamp = adder->offset * GST_SECOND / adder->rate;
+    GST_BUFFER_DURATION (buf_out) = duration;
+
+    adder->offset += delta;
+    adder->timestamp += duration;
 
     /* send it out */
     GST_LOG ("pushing buf_out");
