@@ -94,6 +94,7 @@ mulawenc_link (GstPad * pad, const GstCaps * caps)
   GstStructure *structure;
   const GValue *rate, *chans;
   GstCaps *base_caps;
+  GstPadLinkReturn link_return;
 
   structure = gst_caps_get_structure (caps, 0);
   rate = gst_structure_get_value (structure, "rate");
@@ -116,7 +117,11 @@ mulawenc_link (GstPad * pad, const GstCaps * caps)
   gst_structure_set_value (structure, "rate", rate);
   gst_structure_set_value (structure, "channels", chans);
 
-  return gst_pad_try_set_caps (otherpad, base_caps);
+  link_return = gst_pad_try_set_caps (otherpad, base_caps);
+
+  gst_caps_free (base_caps);
+
+  return link_return;
 }
 
 GType
@@ -203,7 +208,7 @@ gst_mulawenc_chain (GstPad * pad, GstData * _data)
   outbuf = gst_buffer_new_and_alloc (GST_BUFFER_SIZE (buf) / 2);
   GST_BUFFER_TIMESTAMP (outbuf) = GST_BUFFER_TIMESTAMP (buf);
   GST_BUFFER_DURATION (outbuf) = GST_BUFFER_DURATION (buf);
-  mulaw_data = (gint8 *) GST_BUFFER_DATA (outbuf);
+  mulaw_data = (guint8 *) GST_BUFFER_DATA (outbuf);
   mulaw_encode (linear_data, mulaw_data, GST_BUFFER_SIZE (outbuf));
 
   gst_buffer_unref (buf);
