@@ -82,7 +82,6 @@ static GstStaticPadTemplate videosink_templ =
 
 /* FIXME:
  * * audio/x-raw-float: endianness needs defining.
- * * audio/x-vorbis: private data setup needs work.
  */
 static GstStaticPadTemplate audiosink_templ =
     GST_STATIC_PAD_TEMPLATE ("audio_%d",
@@ -97,6 +96,8 @@ static GstStaticPadTemplate audiosink_templ =
         COMMON_AUDIO_CAPS "; "
         "audio/x-ac3, "
         COMMON_AUDIO_CAPS "; "
+        "audio/x-vorbis, "
+        COMMON_AUDIO_CAPS "; "
         "audio/x-raw-int, "
         "width = (int) { 8, 16, 24 }, "
         "depth = (int) { 8, 16, 24 }, "
@@ -321,7 +322,7 @@ gst_matroska_mux_reset (GstElement * element)
 
   /* reset uid array */
   if (used_uids) {
-    g_free (used_uids);
+    g_array_free (used_uids, TRUE);
   }
   /* arbitrary size, 10 should be enough in most cases */
   used_uids = g_array_sized_new (FALSE, FALSE, sizeof (guint32), 10);
@@ -575,7 +576,77 @@ gst_matroska_mux_audio_pad_link (GstPad * pad, const GstCaps * caps)
   } else if (!strcmp (mimetype, "audio/x-raw-float")) {
     /* FIXME: endianness is undefined */
   } else if (!strcmp (mimetype, "audio/x-vorbis")) {
-    /* FIXME: private data setup needs work */
+    const GValue *streamheader;
+    guint8 *priv_data = NULL;
+    guint priv_data_size = 0;
+
+    context->codec_id = g_strdup (GST_MATROSKA_CODEC_ID_AUDIO_VORBIS);
+
+    if (context->codec_priv != NULL) {
+      g_free (context->codec_priv);
+      context->codec_priv = NULL;
+      context->codec_priv_size = 0;
+    }
+    streamheader = gst_structure_get_value (structure, "streamheader");
+
+    if (streamheader != NULL) {
+      if (G_VALUE_TYPE (streamheader) == GST_TYPE_FIXED_LIST) {
+        GArray *bufarr = g_value_peek_pointer (streamheader);
+        gint i;
+        gint offset;
+
+        if (bufarr->len == 3) {
+          GstBuffer *buf[3];
+
+          for (i = 0; i < bufarr->len; i++) {
+            GValue *bufval = &g_array_index (bufarr, GValue, i);
+
+            if (G_VALUE_TYPE (bufval) == GST_TYPE_BUFFER) {
+              buf[i] = g_value_peek_pointer (bufval);
+            }
+          }
+
+          priv_data_size = 1;
+          priv_data_size += GST_BUFFER_SIZE (buf[0]) / 0xff + 1;
+          priv_data_size += GST_BUFFER_SIZE (buf[1]) / 0xff + 1;
+
+          for (i = 0; i < 3; ++i) {
+            priv_data_size += GST_BUFFER_SIZE (buf[i]);
+          }
+
+          priv_data = g_malloc0 (priv_data_size);
+
+          priv_data[0] = 2;
+          offset = 1;
+
+          for (i = 0; i < GST_BUFFER_SIZE (buf[0]) / 0xff; ++i) {
+            priv_data[offset++] = 0xff;
+          }
+          priv_data[offset++] = GST_BUFFER_SIZE (buf[0]) % 0xff;
+
+          for (i = 0; i < GST_BUFFER_SIZE (buf[1]) / 0xff; ++i) {
+            priv_data[offset++] = 0xff;
+          }
+          priv_data[offset++] = GST_BUFFER_SIZE (buf[1]) % 0xff;
+
+          for (i = 0; i < 3; ++i) {
+            memcpy (priv_data + offset, GST_BUFFER_DATA (buf[i]),
+                GST_BUFFER_SIZE (buf[i]));
+            offset += GST_BUFFER_SIZE (buf[i]);
+          }
+        }
+      }
+    }
+
+    if (priv_data == NULL) {
+      /* FIXME:  */
+    }
+
+    context->codec_priv_size = priv_data_size;
+    context->codec_priv = (gpointer) priv_data;
+
+    return GST_PAD_LINK_OK;
+
   } else if (!strcmp (mimetype, "audio/x-ac3")) {
     context->codec_id = g_strdup (GST_MATROSKA_CODEC_ID_AUDIO_AC3);
 
@@ -974,7 +1045,6 @@ gst_matroska_mux_prepare_data (GstMatroskaMux * mux)
 
   for (i = 0; i < mux->num_streams; i++) {
     while (!mux->sink[i].eos && !mux->sink[i].buffer &&
-        mux->sink[i].track->num > 0 &&
         GST_PAD_IS_USABLE (mux->sink[i].track->pad)) {
       GstData *data;
 
@@ -1133,19 +1203,32 @@ gst_matroska_mux_loop (GstElement * element)
   guint i;
 
   if (gst_matroska_mux_prepare_data (mux) == -1) {
-    GST_ELEMENT_ERROR (element, STREAM, MUX, (NULL), ("No data"));
+    gboolean eos = TRUE;
+
+    for (i = 0; i < mux->num_streams; i++) {
+      eos = eos & mux->sink[i].eos;
+    }
+    if (eos) {
+      // all streams are EOS, finish writing
+      gst_matroska_mux_write_data (mux);
+    } else {
+      // no data available and no EOS
+      GST_ELEMENT_ERROR (element, STREAM, MUX, (NULL), ("No data"));
+    }
     return;
   }
 
   /* start with a header */
   if (mux->state == GST_MATROSKA_MUX_STATE_START) {
     if (mux->num_streams == 0) {
+      GST_ELEMENT_ERROR (mux, STREAM, MUX, (NULL),
+          ("No input streams configured"));
       return;
     }
     for (i = 0; i < mux->num_streams; i++) {
       if (!gst_pad_is_negotiated (mux->sink[i].track->pad)) {
+        GST_ELEMENT_ERROR (mux, CORE, NEGOTIATION, (NULL), (NULL));
         return;
-      } else {
       }
     }
     mux->state = GST_MATROSKA_MUX_STATE_HEADER;
