@@ -27,20 +27,14 @@
 #include "matroska-mux.h"
 
 static gboolean
-plugin_init (GstPlugin *plugin)
+plugin_init (GstPlugin * plugin)
 {
   return (gst_matroska_demux_plugin_init (plugin) &&
-	  gst_matroska_mux_plugin_init (plugin));
+      gst_matroska_mux_plugin_init (plugin));
 }
 
-GST_PLUGIN_DEFINE (
-  GST_VERSION_MAJOR,
-  GST_VERSION_MINOR,
-  "matroska",
-  "Matroska stream handling",
-  plugin_init,
-  VERSION,
-  "LGPL",
-  GST_PACKAGE,
-  GST_ORIGIN
-)
+GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
+    GST_VERSION_MINOR,
+    "matroska",
+    "Matroska stream handling",
+    plugin_init, VERSION, "LGPL", GST_PACKAGE, GST_ORIGIN)
