@@ -10,6 +10,11 @@
 // J2B module loader
 //
 ///////////////////////////////////////////////////
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "stdafx.h"
 #include "sndfile.h"
 
