@@ -25,11 +25,7 @@
 #include <gst/gst.h>
 #include "gstmpegdemux.h"
 
-
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
-
+G_BEGIN_DECLS
 
 #define GST_TYPE_DVD_DEMUX \
   (gst_dvd_demux_get_type())
@@ -81,41 +77,44 @@ enum {
    streams. */
 struct _GstDVDLPCMStream {
   GstMPEGStream	 parent;
-  guint8	 sample_info;	/* The type of linear PCM samples
+  guint32	 sample_info;	/* The type of linear PCM samples
                                    associated to this stream. The
                                    values are bit fields with the same
                                    format of the sample_info field in
                                    the linear PCM header. */
-  gint rate, channels, width;
+  gint rate, channels, width,
+    dynamic_range;
+  gboolean mute, emphasis;
 };
 
 
 struct _GstDVDDemux {
   GstMPEGDemux	 parent;
 
-  GstPad *cur_video;		/* Current video stream pad. */
-  GstPad *cur_audio;		/* Current audio stream pad. */
-  GstPad *cur_subpicture;	/* Current subpicture stream pad. */
+  GstPad *cur_video;           /* Current video stream pad. */
+  GstPad *cur_audio;           /* Current audio stream pad. */
+  GstPad *cur_subpicture;      /* Current subpicture stream pad. */
 
   gint cur_video_nr;		/* Current video stream number. */
   gint cur_audio_nr;		/* Current audio stream number. */
   gint cur_subpicture_nr;	/* Current subpicture stream number. */
 
-  GstClockTime last_end_ptm;	/* End presentation time of the las nav packet
+  GstClockTime last_end_ptm;	/* End presentation time of the last nav packet
                                    event received. */
 
-  gboolean just_flushed;	/* The element just received a flush event. */
   GstClockTime discont_time;	/* If different from GST_CLOCK_TIME_NONE, a
                                    discontinuous event should be sent with the
-                                   given time, before sending the next dara
+                                   given time, before sending the next data
                                    block.. */
 
   gint mpeg_version;		/* Version of the MPEG video stream */
 
   GstMPEGStream *subpicture_stream[GST_DVD_DEMUX_NUM_SUBPICTURE_STREAMS];
 				/* Subpicture output streams. */
-  GstClockTime   subpicture_time[GST_DVD_DEMUX_NUM_SUBPICTURE_STREAMS];
-  				/* Last timestamp for buffer on each stream */
+
+  GstEvent *langcodes;
+
+  gboolean ignore_next_newmedia_discont;
 };
 
 
@@ -139,10 +138,6 @@ GType		gst_dvd_demux_get_type		(void);
 
 gboolean 	gst_dvd_demux_plugin_init 	(GstPlugin *plugin);
 
-
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
-
+G_END_DECLS
 
 #endif /* __DVD_DEMUX_H__ */
