@@ -24,14 +24,16 @@
 #include <string.h>
 #include <gst/gst.h>
 
+#include "gstplay-marshal.h"
+
 /* generic templates */
-GstStaticPadTemplate decoder_bin_sink_template =
+static GstStaticPadTemplate decoder_bin_sink_template =
 GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS_ANY);
 
-GstStaticPadTemplate decoder_bin_src_template =
+static GstStaticPadTemplate decoder_bin_src_template =
 GST_STATIC_PAD_TEMPLATE ("src%d",
     GST_PAD_SRC,
     GST_PAD_SOMETIMES,
@@ -72,8 +74,12 @@ struct _GstDecodeBinClass
 
   /* signal we fire when a new pad has been decoded into raw audio/video */
   void (*new_decoded_pad) (GstElement * element, GstPad * pad, gboolean last);
+  /* signal we fire when a pad has been removed */
+  void (*removed_decoded_pad) (GstElement * element, GstPad * pad);
   /* signal fired when we found a pad that we cannot decode */
-  void (*unknown_type) (GstElement * element, GstCaps * caps);
+  void (*unknown_type) (GstElement * element, GstPad * pad, GstCaps * caps);
+  /* signal fired when we got a redirect attempt */
+  void (*got_redirect) (GstElement * element, const gchar * new_location);
 };
 
 /* props */
@@ -87,7 +93,9 @@ enum
 enum
 {
   SIGNAL_NEW_DECODED_PAD,
+  SIGNAL_REMOVED_DECODED_PAD,
   SIGNAL_UNKNOWN_TYPE,
+  SIGNAL_REDIRECT,
   LAST_SIGNAL
 };
 
@@ -96,6 +104,7 @@ enum
 typedef struct
 {
   gint np_sig_id;               /* signal id of new_pad */
+  gint unlink_sig_id;           /* signal id of unlinked */
   gint nmp_sig_id;              /* signal id of no_more_pads */
   GstElement *element;          /* the element sending the signal */
   GstDecodeBin *decode_bin;     /* pointer to ourself */
@@ -119,6 +128,8 @@ static GstElement *try_to_link_1 (GstDecodeBin * decode_bin, GstPad * pad,
 static void close_link (GstElement * element, GstDecodeBin * decode_bin);
 static void close_pad_link (GstElement * element, GstPad * pad,
     GstCaps * caps, GstDecodeBin * decode_bin, gboolean more);
+static void unlinked (GstPad * pad, GstPad * peerpad,
+    GstDecodeBin * decode_bin);
 
 static GstElementClass *parent_class;
 static guint gst_decode_bin_signals[LAST_SIGNAL] = { 0 };
@@ -182,15 +193,28 @@ gst_decode_bin_class_init (GstDecodeBinClass * klass)
       g_signal_new ("new-decoded-pad", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST,
       G_STRUCT_OFFSET (GstDecodeBinClass, new_decoded_pad), NULL, NULL,
-      gst_marshal_VOID__OBJECT_POINTER, G_TYPE_NONE, 2, G_TYPE_OBJECT,
+      gst_play_marshal_VOID__OBJECT_BOOLEAN, G_TYPE_NONE, 2, GST_TYPE_PAD,
       G_TYPE_BOOLEAN);
+  gst_decode_bin_signals[SIGNAL_REMOVED_DECODED_PAD] =
+      g_signal_new ("removed-decoded-pad", G_TYPE_FROM_CLASS (klass),
+      G_SIGNAL_RUN_LAST,
+      G_STRUCT_OFFSET (GstDecodeBinClass, removed_decoded_pad), NULL, NULL,
+      gst_marshal_VOID__OBJECT, G_TYPE_NONE, 1, GST_TYPE_PAD);
   gst_decode_bin_signals[SIGNAL_UNKNOWN_TYPE] =
       g_signal_new ("unknown-type", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GstDecodeBinClass, unknown_type),
-      NULL, NULL, gst_marshal_VOID__OBJECT, G_TYPE_NONE, 1, GST_TYPE_CAPS);
+      NULL, NULL, gst_marshal_VOID__OBJECT_BOXED, G_TYPE_NONE, 2,
+      GST_TYPE_PAD, GST_TYPE_CAPS);
+  gst_decode_bin_signals[SIGNAL_REDIRECT] =
+      g_signal_new ("got-redirect", G_TYPE_FROM_CLASS (klass),
+      G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GstDecodeBinClass, got_redirect),
+      NULL, NULL, g_cclosure_marshal_VOID__STRING, G_TYPE_NONE, 1,
+      G_TYPE_STRING);
 
   gobject_klass->dispose = GST_DEBUG_FUNCPTR (gst_decode_bin_dispose);
 
+  gst_element_class_add_pad_template (gstelement_klass,
+      gst_static_pad_template_get (&decoder_bin_sink_template));
   gst_element_class_add_pad_template (gstelement_klass,
       gst_static_pad_template_get (&decoder_bin_src_template));
 
@@ -226,7 +250,8 @@ gst_decode_bin_factory_filter (GstPluginFeature * feature,
 
   klass = gst_element_factory_get_klass (GST_ELEMENT_FACTORY (feature));
   /* only demuxers and decoders can play */
-  if (strstr (klass, "Demux") == NULL && strstr (klass, "Decoder") == NULL) {
+  if (strstr (klass, "Demux") == NULL &&
+      strstr (klass, "Decoder") == NULL && strstr (klass, "Parse") == NULL) {
     return FALSE;
   }
 
@@ -242,7 +267,13 @@ gst_decode_bin_factory_filter (GstPluginFeature * feature,
 static gint
 compare_ranks (GstPluginFeature * f1, GstPluginFeature * f2)
 {
-  return gst_plugin_feature_get_rank (f2) - gst_plugin_feature_get_rank (f1);
+  gint diff;
+
+  diff = gst_plugin_feature_get_rank (f2) - gst_plugin_feature_get_rank (f1);
+  if (diff != 0)
+    return diff;
+  return strcmp (gst_plugin_feature_get_name (f2),
+      gst_plugin_feature_get_name (f1));
 }
 
 static void
@@ -347,6 +378,8 @@ find_compatibles (GstDecodeBin * decode_bin, const GstCaps * caps)
         if (!gst_caps_is_empty (intersect)) {
           /* non empty intersection, we can use this element */
           to_try = g_list_append (to_try, factory);
+          gst_caps_free (intersect);
+          break;
         }
         gst_caps_free (intersect);
       }
@@ -372,11 +405,14 @@ close_pad_link (GstElement * element, GstPad * pad, GstCaps * caps,
   GstStructure *structure;
   const gchar *mimetype;
 
+  if (!strncmp (gst_pad_get_name (pad), "current_", 8))
+    return;
+
   /* the caps is empty, this means the pad has no type, we can only
    * decide to fire the unknown_type signal. */
-  if (gst_caps_is_empty (caps)) {
+  if (caps == NULL || gst_caps_is_empty (caps)) {
     g_signal_emit (G_OBJECT (decode_bin),
-        gst_decode_bin_signals[SIGNAL_UNKNOWN_TYPE], 0, caps);
+        gst_decode_bin_signals[SIGNAL_UNKNOWN_TYPE], 0, pad, caps);
     return;
   }
 
@@ -388,14 +424,17 @@ close_pad_link (GstElement * element, GstPad * pad, GstCaps * caps,
 
   GST_LOG_OBJECT (element, "trying to close %" GST_PTR_FORMAT, caps);
 
-  /* FIXME, iterate over more structures? */
+  /* FIXME, iterate over more structures? I guess it is possible that
+   * this pad has some encoded and some raw pads. This code will fail
+   * then if the first structure is not the raw type... */
   structure = gst_caps_get_structure (caps, 0);
   mimetype = gst_structure_get_name (structure);
 
   /* first see if this is raw. If the type is raw, we can
    * create a ghostpad for this pad. */
   if (g_str_has_prefix (mimetype, "video/x-raw") ||
-      g_str_has_prefix (mimetype, "audio/x-raw")) {
+      g_str_has_prefix (mimetype, "audio/x-raw") ||
+      g_str_has_prefix (mimetype, "text/plain")) {
     gchar *padname;
     GstPad *ghost;
 
@@ -423,15 +462,24 @@ close_pad_link (GstElement * element, GstPad * pad, GstCaps * caps,
       /* no compatible elements, fire the unknown_type signal, we cannot go
        * on */
       g_signal_emit (G_OBJECT (decode_bin),
-          gst_decode_bin_signals[SIGNAL_UNKNOWN_TYPE], 0, caps);
+          gst_decode_bin_signals[SIGNAL_UNKNOWN_TYPE], 0, pad, caps);
       return;
     }
     try_to_link_1 (decode_bin, pad, to_try);
   } else {
     GST_LOG_OBJECT (element, "multiple possibilities, delaying");
-    g_warning ("multiple possibilities, delaying");
   }
+}
 
+/*
+ * Called when we're redirected to a new URI.
+ */
+static void
+got_redirect (GstElement * element,
+    const gchar * new_location, GstDecodeBin * decode_bin)
+{
+  g_signal_emit (decode_bin, gst_decode_bin_signals[SIGNAL_REDIRECT], 0,
+      new_location);
 }
 
 /* given a list of element factories, try to link one of the factories
@@ -453,7 +501,7 @@ try_to_link_1 (GstDecodeBin * decode_bin, GstPad * pad, GList * factories)
     /* make an element from the factory first */
     element = gst_element_factory_create (factory, NULL);
     if (element == NULL) {
-      /* hmm, strange */
+      /* hmm, strange. Like with all things in live, let's move on.. */
       GST_WARNING_OBJECT (decode_bin, "could not create  an element from %s",
           gst_plugin_feature_get_name (GST_PLUGIN_FEATURE (factory)));
       continue;
@@ -463,6 +511,7 @@ try_to_link_1 (GstDecodeBin * decode_bin, GstPad * pad, GList * factories)
     GST_DEBUG_OBJECT (decode_bin, "adding %s", gst_element_get_name (element));
     gst_bin_add (GST_BIN (decode_bin), element);
 
+    /* set to ready first so it can do negotiation */
     gst_element_set_state (element, GST_STATE_READY);
 
     /* keep out own list of elements */
@@ -475,6 +524,10 @@ try_to_link_1 (GstDecodeBin * decode_bin, GstPad * pad, GList * factories)
     if (ret) {
       const gchar *klass;
       GstElementFactory *factory;
+      guint sig;
+
+      GST_DEBUG_OBJECT (decode_bin, "linked on pad %s:%s",
+          GST_DEBUG_PAD_NAME (pad));
 
       /* The link worked, now figure out what it was that we connected */
       factory = gst_element_get_factory (element);
@@ -482,9 +535,23 @@ try_to_link_1 (GstDecodeBin * decode_bin, GstPad * pad, GList * factories)
       /* check if we can use threads */
       if (decode_bin->threaded) {
         if (strstr (klass, "Demux") != NULL) {
-          /* FIXME, do something with threads here */
+          /* FIXME, do something with threads here. Not sure that it 
+           * really matters here but in general it is better to preroll
+           * on encoded data from the muxer than on raw encoded streams
+           * because that would consume less memory. */
         }
       }
+      /* catch redirects */
+      if (g_signal_lookup ("got-redirect", G_OBJECT_TYPE (element))) {
+        g_signal_connect (element, "got-redirect",
+            G_CALLBACK (got_redirect), decode_bin);
+      }
+
+      /* make sure we catch unlink signals */
+      sig = g_signal_connect (G_OBJECT (GST_PAD_REALIZE (pad)), "unlinked",
+          G_CALLBACK (unlinked), decode_bin);
+      /* keep a ref to the signal id so that we can disconnect the signal callback */
+      g_object_set_data (G_OBJECT (pad), "unlinked_id", GINT_TO_POINTER (sig));
 
       /* now that we added the element we can try to continue autoplugging
        * on it until we have a raw type */
@@ -493,6 +560,8 @@ try_to_link_1 (GstDecodeBin * decode_bin, GstPad * pad, GList * factories)
       gst_element_sync_state_with_parent (element);
       return element;
     } else {
+      GST_DEBUG_OBJECT (decode_bin, "link failed on pad %s:%s",
+          GST_DEBUG_PAD_NAME (pad));
       /* this element did not work, remove it again and continue trying
        * other elements */
       gst_bin_remove (GST_BIN (decode_bin), element);
@@ -501,17 +570,123 @@ try_to_link_1 (GstDecodeBin * decode_bin, GstPad * pad, GList * factories)
   return NULL;
 }
 
+static GstPad *
+get_our_ghost_pad (GstDecodeBin * decode_bin, GstPad * pad)
+{
+  GList *ghostpads;
+
+  if (pad == NULL || !GST_PAD_IS_SRC (pad)) {
+    GST_DEBUG_OBJECT (decode_bin, "pad NULL or not SRC pad");
+    return NULL;
+  }
+
+  if (GST_IS_GHOST_PAD (pad)) {
+    GstElement *parent = gst_pad_get_parent (pad);
+
+    GST_DEBUG_OBJECT (decode_bin, "pad parent %s",
+        gst_element_get_name (parent));
+    if (parent == GST_ELEMENT (decode_bin)) {
+      GST_DEBUG_OBJECT (decode_bin, "pad is our ghostpad");
+      return pad;
+    } else {
+      GST_DEBUG_OBJECT (decode_bin, "pad is ghostpad but not ours");
+      return NULL;
+    }
+  }
+
+  GST_DEBUG_OBJECT (decode_bin, "looping over ghostpads");
+  ghostpads = gst_pad_get_ghost_pad_list (pad);
+  while (ghostpads) {
+    GstPad *ghostpad;
+
+    ghostpad = get_our_ghost_pad (decode_bin, GST_PAD (ghostpads->data));
+    if (ghostpad)
+      return ghostpad;
+
+    ghostpads = g_list_next (ghostpads);
+  }
+  GST_DEBUG_OBJECT (decode_bin, "done looping over ghostpads, nothing found");
+
+  return NULL;
+}
+
+/* remove all downstream elements starting from the given pad.
+ * Also make sure to remove the ghostpad we created for the raw 
+ * decoded stream.
+ */
+static void
+remove_element_chain (GstDecodeBin * decode_bin, GstPad * pad)
+{
+  GList *int_links;
+  GstElement *elem = gst_pad_get_parent (pad);
+
+  while (GST_OBJECT_PARENT (elem) &&
+      GST_OBJECT_PARENT (elem) != GST_OBJECT (decode_bin))
+    elem = GST_ELEMENT (GST_OBJECT_PARENT (elem));
+
+  GST_DEBUG_OBJECT (decode_bin, "%s:%s", GST_DEBUG_PAD_NAME (pad));
+
+  /* remove all elements linked to this pad up to the ghostpad 
+   * that we created for this stream */
+  for (int_links = gst_pad_get_internal_links (pad);
+      int_links; int_links = g_list_next (int_links)) {
+    GstPad *pad;
+    GstPad *ghostpad;
+    GstPad *peer;
+
+    pad = GST_PAD (int_links->data);
+    GST_DEBUG_OBJECT (decode_bin, "inspecting internal pad %s:%s",
+        GST_DEBUG_PAD_NAME (pad));
+
+    ghostpad = get_our_ghost_pad (decode_bin, pad);
+    if (ghostpad) {
+      GST_DEBUG_OBJECT (decode_bin, "found our ghost pad %s:%s for %s:%s",
+          GST_DEBUG_PAD_NAME (ghostpad), GST_DEBUG_PAD_NAME (pad));
+
+      g_signal_emit (G_OBJECT (decode_bin),
+          gst_decode_bin_signals[SIGNAL_REMOVED_DECODED_PAD], 0, ghostpad);
+
+      gst_element_remove_pad (GST_ELEMENT (decode_bin), ghostpad);
+      continue;
+    } else {
+      GST_DEBUG_OBJECT (decode_bin, "not one of our ghostpads");
+    }
+
+    peer = gst_pad_get_peer (pad);
+    if (peer == NULL)
+      continue;
+
+    GST_DEBUG_OBJECT (decode_bin, "internal pad %s:%s linked to pad %s:%s",
+        GST_DEBUG_PAD_NAME (pad), GST_DEBUG_PAD_NAME (peer));
+
+    if (gst_pad_get_real_parent (peer) != GST_ELEMENT (decode_bin)) {
+      GST_DEBUG_OBJECT (decode_bin, "dead end pad %s:%s",
+          GST_DEBUG_PAD_NAME (peer));
+    } else {
+      GST_DEBUG_OBJECT (decode_bin, "recursing element %s on pad %s:%s",
+          gst_element_get_name (elem), GST_DEBUG_PAD_NAME (pad));
+      remove_element_chain (decode_bin, peer);
+    }
+  }
+  GST_DEBUG_OBJECT (decode_bin, "removing %s", gst_element_get_name (elem));
+  gst_bin_remove (GST_BIN (decode_bin), elem);
+}
+
 /* This function will be called when a dynamic pad is created on an element.
  * We try to continue autoplugging on this new pad. */
 static void
 new_pad (GstElement * element, GstPad * pad, GstDynamic * dynamic)
 {
   GstDecodeBin *decode_bin = dynamic->decode_bin;
+  GstCaps *caps;
 
   /* see if any more pending dynamic connections exist */
   gboolean more = gst_decode_bin_is_dynamic (decode_bin);
 
-  close_pad_link (element, pad, gst_pad_get_caps (pad), decode_bin, more);
+  caps = gst_pad_get_caps (pad);
+  close_pad_link (element, pad, caps, decode_bin, more);
+  if (caps)
+    gst_caps_free (caps);
 }
 
 /* this signal is fired when an element signals the no_more_pads signal.
@@ -536,7 +711,7 @@ no_more_pads (GstElement * element, GstDynamic * dynamic)
   decode_bin->dynamics = g_list_remove (decode_bin->dynamics, dynamic);
   g_free (dynamic);
 
-  /* if we have no more dynamic elements, we have no change of creating
+  /* if we have no more dynamic elements, we have no chance of creating
    * more pads, so we fire the no_more_pads signal */
   if (decode_bin->dynamics == NULL) {
     GST_DEBUG_OBJECT (decode_bin,
@@ -547,6 +722,43 @@ no_more_pads (GstElement * element, GstDynamic * dynamic)
   }
 }
 
+/* This function will be called when a pad is disconnected for some reason */
+static void
+unlinked (GstPad * pad, GstPad * peerpad, GstDecodeBin * decode_bin)
+{
+  GList *walk;
+  GstDynamic *dyn;
+  GstElement *element;
+
+  /* inactivate pad */
+  gst_pad_set_active (pad, FALSE);
+
+  /* remove all elements linked to the peerpad */
+  remove_element_chain (decode_bin, peerpad);
+
+  /* if an element removes two pads, then we don't want this twice */
+  element = gst_pad_get_parent (pad);
+  for (walk = decode_bin->dynamics; walk != NULL; walk = walk->next) {
+    dyn = walk->data;
+    if (dyn->element == element)
+      return;
+  }
+
+  GST_DEBUG_OBJECT (decode_bin, "pad removal while alive - chained?");
+
+  /* re-setup dynamic plugging */
+  dyn = g_new0 (GstDynamic, 1);
+  dyn->np_sig_id = g_signal_connect (G_OBJECT (element), "new-pad",
+      G_CALLBACK (new_pad), dyn);
+  dyn->nmp_sig_id = g_signal_connect (G_OBJECT (element), "no-more-pads",
+      G_CALLBACK (no_more_pads), dyn);
+  dyn->element = element;
+  dyn->decode_bin = decode_bin;
+
+  /* and add this element to the dynamic elements */
+  decode_bin->dynamics = g_list_prepend (decode_bin->dynamics, dyn);
+}
+
 /* this function inspects the given element and tries to connect something
  * on the srcpads. If there are dynamic pads, it sets up a signal handler to
  * continue autoplugging when they become available */
@@ -646,6 +858,7 @@ close_link (GstElement * element, GstDecodeBin * decode_bin)
   /* now loop over all the pads we need to connect */
   for (pads = to_connect; pads; pads = g_list_next (pads)) {
     GstPad *pad = GST_PAD (pads->data);
+    GstCaps *caps;
 
     /* we have more pads if we have more than 1 pad to connect or
      * dynamics. If we have only 1 pad and no dynamics, more will be
@@ -657,7 +870,10 @@ close_link (GstElement * element, GstDecodeBin * decode_bin)
         gst_pad_get_name (pad));
 
     /* continue autoplugging on the pads */
-    close_pad_link (element, pad, gst_pad_get_caps (pad), decode_bin, more);
+    caps = gst_pad_get_caps (pad);
+    close_pad_link (element, pad, caps, decode_bin, more);
+    if (caps)
+      gst_caps_free (caps);
   }
 
   g_list_free (to_connect);
