@@ -232,7 +232,7 @@ plugin_init (GstPlugin * plugin)
     return FALSE;
 
   return gst_element_register (plugin, "rmdemux",
-      GST_RANK_PRIMARY, GST_TYPE_RMDEMUX);
+      GST_RANK_NONE, GST_TYPE_RMDEMUX);
 }
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
@@ -693,7 +693,7 @@ re_get_pascal_string (guint8 * ptr)
   int length;
 
   length = ptr[0];
-  return g_strndup (ptr + 1, length);
+  return g_strndup ((char *) ptr + 1, length);
 }
 
 static int
