@@ -29,11 +29,10 @@
 #include <sys/filio.h>
 #endif
 
+#include "gsttcp.h"
 #include "gsttcpserversink.h"
 #include "gsttcp-marshal.h"
 
-#define TCP_DEFAULT_HOST	"127.0.0.1"
-#define TCP_DEFAULT_PORT	4953
 #define TCP_BACKLOG		5
 
 /* elementfactory information */
@@ -56,11 +55,13 @@ enum
 static void gst_tcpserversink_base_init (gpointer g_class);
 static void gst_tcpserversink_class_init (GstTCPServerSink * klass);
 static void gst_tcpserversink_init (GstTCPServerSink * tcpserversink);
+static void gst_tcpserversink_finalize (GObject * gobject);
 
 static gboolean gst_tcpserversink_handle_wait (GstMultiFdSink * sink,
     GstFDSet * set);
 static gboolean gst_tcpserversink_init_send (GstMultiFdSink * this);
 static gboolean gst_tcpserversink_close (GstMultiFdSink * this);
+static void gst_tcpserversink_removed (GstMultiFdSink * sink, int fd);
 
 static void gst_tcpserversink_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
@@ -118,19 +119,21 @@ gst_tcpserversink_class_init (GstTCPServerSink * klass)
 
   parent_class = g_type_class_ref (GST_TYPE_MULTIFDSINK);
 
+  gobject_class->set_property = gst_tcpserversink_set_property;
+  gobject_class->get_property = gst_tcpserversink_get_property;
+  gobject_class->finalize = gst_tcpserversink_finalize;
+
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_HOST,
       g_param_spec_string ("host", "host", "The host/IP to send the packets to",
           TCP_DEFAULT_HOST, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_PORT,
       g_param_spec_int ("port", "port", "The port to send the packets to",
-          0, 32768, TCP_DEFAULT_PORT, G_PARAM_READWRITE));
-
-  gobject_class->set_property = gst_tcpserversink_set_property;
-  gobject_class->get_property = gst_tcpserversink_get_property;
+          0, TCP_HIGHEST_PORT, TCP_DEFAULT_PORT, G_PARAM_READWRITE));
 
   gstmultifdsink_class->init = gst_tcpserversink_init_send;
   gstmultifdsink_class->wait = gst_tcpserversink_handle_wait;
   gstmultifdsink_class->close = gst_tcpserversink_close;
+  gstmultifdsink_class->removed = gst_tcpserversink_removed;
 
   GST_DEBUG_CATEGORY_INIT (tcpserversink_debug, "tcpserversink", 0, "TCP sink");
 }
@@ -141,10 +144,19 @@ gst_tcpserversink_init (GstTCPServerSink * this)
   this->server_port = TCP_DEFAULT_PORT;
   /* should support as minimum 576 for IPV4 and 1500 for IPV6 */
   /* this->mtu = 1500; */
+  this->host = g_strdup (TCP_DEFAULT_HOST);
 
   this->server_sock.fd = -1;
 }
 
+static void
+gst_tcpserversink_finalize (GObject * gobject)
+{
+  GstTCPServerSink *this = GST_TCPSERVERSINK (gobject);
+
+  g_free (this->host);
+}
+
 /* handle a read request on the server,
  * which indicates a new client connection */
 static gboolean
@@ -153,14 +165,20 @@ gst_tcpserversink_handle_server_read (GstTCPServerSink * sink)
   /* new client */
   int client_sock_fd;
   struct sockaddr_in client_address;
-  int client_address_len;
+  unsigned int client_address_len;
+
+  /* For some stupid reason, client_address and client_address_len has to be
+   * zeroed */
+  memset (&client_address, 0, sizeof (client_address));
+  client_address_len = 0;
 
   client_sock_fd =
       accept (sink->server_sock.fd, (struct sockaddr *) &client_address,
       &client_address_len);
   if (client_sock_fd == -1) {
     GST_ELEMENT_ERROR (sink, RESOURCE, OPEN_WRITE, (NULL),
-        ("Could not accept client on server socket: %s", g_strerror (errno)));
+        ("Could not accept client on server socket %d: %s (%d)",
+            sink->server_sock.fd, g_strerror (errno), errno));
     return FALSE;
   }
 
@@ -172,6 +190,18 @@ gst_tcpserversink_handle_server_read (GstTCPServerSink * sink)
   return TRUE;
 }
 
+static void
+gst_tcpserversink_removed (GstMultiFdSink * sink, int fd)
+{
+  GstTCPServerSink *this = GST_TCPSERVERSINK (sink);
+
+  GST_LOG_OBJECT (this, "closing fd %d", fd);
+  if (close (fd) < 0) {
+    GST_WARNING_OBJECT (this, "error closing fd %d: %s", fd,
+        g_strerror (errno));
+  }
+}
+
 static gboolean
 gst_tcpserversink_handle_wait (GstMultiFdSink * sink, GstFDSet * set)
 {
@@ -199,6 +229,10 @@ gst_tcpserversink_set_property (GObject * object, guint prop_id,
 
   switch (prop_id) {
     case ARG_HOST:
+      if (!g_value_get_string (value)) {
+        g_warning ("host property cannot be NULL");
+        break;
+      }
       g_free (sink->host);
       sink->host = g_strdup (g_value_get_string (value));
       break;
@@ -252,15 +286,19 @@ gst_tcpserversink_init_send (GstMultiFdSink * parent)
       this->server_sock.fd);
 
   /* make address reusable */
-  if (setsockopt (this->server_sock.fd, SOL_SOCKET, SO_REUSEADDR, &ret,
-          sizeof (int)) < 0) {
+  ret = 1;
+  if (setsockopt (this->server_sock.fd, SOL_SOCKET, SO_REUSEADDR,
+          (void *) &ret, sizeof (ret)) < 0) {
+    gst_tcp_socket_close (&this->server_sock.fd);
     GST_ELEMENT_ERROR (this, RESOURCE, SETTINGS, (NULL),
         ("Could not setsockopt: %s", g_strerror (errno)));
     return FALSE;
   }
   /* keep connection alive; avoids SIGPIPE during write */
-  if (setsockopt (this->server_sock.fd, SOL_SOCKET, SO_KEEPALIVE, &ret,
-          sizeof (int)) < 0) {
+  ret = 1;
+  if (setsockopt (this->server_sock.fd, SOL_SOCKET, SO_KEEPALIVE,
+          (void *) &ret, sizeof (ret)) < 0) {
+    gst_tcp_socket_close (&this->server_sock.fd);
     GST_ELEMENT_ERROR (this, RESOURCE, SETTINGS, (NULL),
         ("Could not setsockopt: %s", g_strerror (errno)));
     return FALSE;
@@ -278,10 +316,12 @@ gst_tcpserversink_init_send (GstMultiFdSink * parent)
       sizeof (this->server_sin));
 
   if (ret) {
+    gst_tcp_socket_close (&this->server_sock.fd);
     switch (errno) {
       default:
         GST_ELEMENT_ERROR (this, RESOURCE, OPEN_READ, (NULL),
-            ("bind failed: %s", g_strerror (errno)));
+            ("bind on port %d failed: %s", this->server_port,
+                g_strerror (errno)));
         return FALSE;
         break;
     }
@@ -293,6 +333,7 @@ gst_tcpserversink_init_send (GstMultiFdSink * parent)
   GST_DEBUG_OBJECT (this, "listening on server socket %d with queue of %d",
       this->server_sock.fd, TCP_BACKLOG);
   if (listen (this->server_sock.fd, TCP_BACKLOG) == -1) {
+    gst_tcp_socket_close (&this->server_sock.fd);
     GST_ELEMENT_ERROR (this, RESOURCE, OPEN_READ, (NULL),
         ("Could not listen on server socket: %s", g_strerror (errno)));
     return FALSE;
@@ -304,8 +345,6 @@ gst_tcpserversink_init_send (GstMultiFdSink * parent)
   gst_fdset_add_fd (parent->fdset, &this->server_sock);
   gst_fdset_fd_ctl_read (parent->fdset, &this->server_sock, TRUE);
 
-  //FD_SET (this->server_sock_fd, &parent->readfds);
-
   return TRUE;
 }
 
@@ -315,10 +354,10 @@ gst_tcpserversink_close (GstMultiFdSink * parent)
   GstTCPServerSink *this = GST_TCPSERVERSINK (parent);
 
   if (this->server_sock.fd != -1) {
+    gst_fdset_remove_fd (parent->fdset, &this->server_sock);
+
     close (this->server_sock.fd);
     this->server_sock.fd = -1;
-
-    gst_fdset_remove_fd (parent->fdset, &this->server_sock);
   }
   return TRUE;
 }
