@@ -1,8 +1,9 @@
 /* GStreamer
  * Copyright (C) 1999,2000 Erik Walthinsen <omega@cse.ogi.edu>
  *                    2000 Wim Taymans <wim.taymans@chello.be>
+ *                    2004 Toni Willberg <toniw@iki.fi>
  *
- * gstosssink.c: 
+ * gstosselement.c:
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -72,7 +73,7 @@ static void gst_osselement_class_init (GstOssElementClass * klass);
 
 static void gst_ossprobe_interface_init (GstPropertyProbeInterface * iface);
 static void gst_osselement_init (GstOssElement * oss);
-static void gst_osselement_dispose (GObject * object);
+static void gst_osselement_finalize (GObject * object);
 
 static void gst_osselement_set_property (GObject * object,
     guint prop_id, const GValue * value, GParamSpec * pspec);
@@ -164,7 +165,7 @@ gst_osselement_class_init (GstOssElementClass * klass)
 
   gobject_class->set_property = gst_osselement_set_property;
   gobject_class->get_property = gst_osselement_get_property;
-  gobject_class->dispose = gst_osselement_dispose;
+  gobject_class->finalize = gst_osselement_finalize;
 
   gstelement_class->change_state = gst_osselement_change_state;
 }
@@ -244,7 +245,7 @@ gst_osselement_class_probe_devices (GstOssElementClass * klass, gboolean check)
   static GList *device_combinations;
   GList *padtempllist;
   gint openmode = O_RDONLY;
-  gboolean mixer = FALSE;
+  gboolean is_mixer = FALSE;
 
   /* Ok, so how do we open the device? We assume that we have (max.) one
    * pad, and if this is a sinkpad, we're osssink (w). else, we're osssrc
@@ -256,7 +257,7 @@ gst_osselement_class_probe_devices (GstOssElementClass * klass, gboolean check)
     if (GST_PAD_TEMPLATE_DIRECTION (firstpadtempl) == GST_PAD_SINK) {
       openmode = O_WRONLY;
     }
-    mixer = TRUE;
+    is_mixer = TRUE;
   }
 
   if (!init && !check) {
@@ -302,7 +303,7 @@ gst_osselement_class_probe_devices (GstOssElementClass * klass, gboolean check)
         /* we just check the dsp. we assume the mixer always works.
          * we don't need a mixer anyway (says OSS)... If we are a
          * mixer element, we use the mixer anyway. */
-        if ((fd = open (mixer ? mixer :
+        if ((fd = open (is_mixer ? mixer :
                     dsp, openmode | O_NONBLOCK)) > 0 || errno == EBUSY) {
           GstOssDeviceCombination *combi;
 
@@ -313,7 +314,7 @@ gst_osselement_class_probe_devices (GstOssElementClass * klass, gboolean check)
           combi = g_new0 (GstOssDeviceCombination, 1);
           combi->dsp = dsp;
           combi->mixer = mixer;
-          combi->dev = mixer ? mixer_dev : dsp_dev;
+          combi->dev = is_mixer ? mixer_dev : dsp_dev;
           device_combinations = device_combination_append (device_combinations,
               combi);
         } else {
@@ -434,14 +435,14 @@ gst_osselement_init (GstOssElement * oss)
 }
 
 static void
-gst_osselement_dispose (GObject * object)
+gst_osselement_finalize (GObject * object)
 {
   GstOssElement *oss = (GstOssElement *) object;
 
   g_free (oss->device);
   g_free (oss->mixer_dev);
 
-  G_OBJECT_CLASS (parent_class)->dispose (object);
+  G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 void
@@ -1060,15 +1061,31 @@ gst_osselement_probe_caps (GstOssElement * oss)
   GstStructure *structure;
   unsigned int format_bit;
   unsigned int format_mask;
+
   GstCaps *caps;
 
+  gboolean mono_supported = FALSE;
+  gboolean stereo_supported = FALSE;
+  int n_channels;
+
   if (oss->probed_caps != NULL)
     return;
   if (oss->fd == -1)
     return;
 
+
   /* FIXME test make sure we're not currently playing */
-  /* FIXME test both mono and stereo */
+
+  /* check if the device supports mono, stereo or both */
+  n_channels = 1;
+  ret = ioctl (oss->fd, SNDCTL_DSP_CHANNELS, &n_channels);
+  if (n_channels == 1)
+    mono_supported = TRUE;
+
+  n_channels = 2;
+  ret = ioctl (oss->fd, SNDCTL_DSP_CHANNELS, &n_channels);
+  if (n_channels == 2)
+    stereo_supported = TRUE;
 
   format_mask = AFMT_U8 | AFMT_S16_LE | AFMT_S16_BE | AFMT_S8 |
       AFMT_U16_LE | AFMT_U16_BE;
@@ -1084,7 +1101,12 @@ gst_osselement_probe_caps (GstOssElement * oss)
       probe = g_new0 (GstOssProbe, 1);
       probe->fd = oss->fd;
       probe->format = format_bit;
-      probe->n_channels = 2;
+
+      if (stereo_supported) {
+        probe->n_channels = 2;
+      } else {
+        probe->n_channels = 1;
+      }
 
       ret = gst_osselement_rate_probe_check (probe);
       if (probe->min == -1 || probe->max == -1) {
@@ -1118,7 +1140,21 @@ gst_osselement_probe_caps (GstOssElement * oss)
       g_free (probe);
 
       structure = gst_osselement_get_format_structure (format_bit);
-      gst_structure_set (structure, "channels", GST_TYPE_INT_RANGE, 1, 2, NULL);
+
+      if (mono_supported && stereo_supported) {
+        gst_structure_set (structure, "channels", GST_TYPE_INT_RANGE, 1, 2,
+            NULL);
+      } else if (mono_supported) {
+        gst_structure_set (structure, "channels", G_TYPE_INT, 1, NULL);
+      } else if (stereo_supported) {
+        gst_structure_set (structure, "channels", G_TYPE_INT, 2, NULL);
+      } else {
+        /* falling back to [1,2] because we don't know what breaks if we abort here */
+        GST_ERROR (_("Your OSS device doesn't support mono or stereo."));
+        gst_structure_set (structure, "channels", GST_TYPE_INT_RANGE, 1, 2,
+            NULL);
+      }
+
       gst_structure_set_value (structure, "rate", &rate_value);
       g_value_unset (&rate_value);
 
