@@ -28,6 +28,8 @@
 /* Object header */
 #include "ximagesink.h"
 
+#include "ximageutil.h"
+
 /* Debugging category */
 #include <gst/gstinfo.h>
 GST_DEBUG_CATEGORY_STATIC (gst_debug_ximagesink);
@@ -64,7 +66,7 @@ GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("video/x-raw-rgb, "
-        "framerate = (double) [ 1.0, 100.0 ], "
+        "framerate = (double) [ 1.0, 1000.0 ], "
         "width = (int) [ 1, MAX ], " "height = (int) [ 1, MAX ]")
     );
 
@@ -78,7 +80,6 @@ enum
 };
 
 static GstVideoSinkClass *parent_class = NULL;
-static gboolean error_caught = FALSE;
 
 /* ============================================================= */
 /*                                                               */
@@ -88,183 +89,23 @@ static gboolean error_caught = FALSE;
 
 /* X11 stuff */
 
-static int
-gst_ximagesink_handle_xerror (Display * display, XErrorEvent * xevent)
-{
-  char error_msg[1024];
-
-  XGetErrorText (display, xevent->error_code, error_msg, 1024);
-  GST_DEBUG ("ximagesink failed to use XShm calls. error: %s", error_msg);
-  error_caught = TRUE;
-  return 0;
-}
-
-/* This function checks that it is actually really possible to create an image
-   using XShm */
-static gboolean
-gst_ximagesink_check_xshm_calls (GstXContext * xcontext)
-{
-#ifndef HAVE_XSHM
-  return FALSE;
-#else
-  GstXImage *ximage = NULL;
-  int (*handler) (Display *, XErrorEvent *);
-  gboolean result = FALSE;
-
-  g_return_val_if_fail (xcontext != NULL, FALSE);
-
-  ximage = g_new0 (GstXImage, 1);
-  g_return_val_if_fail (ximage != NULL, FALSE);
-
-  /* Setting an error handler to catch failure */
-  error_caught = FALSE;
-  handler = XSetErrorHandler (gst_ximagesink_handle_xerror);
-
-  /* Trying to create a 1x1 picture */
-  GST_DEBUG ("XShmCreateImage of 1x1");
-
-  ximage->ximage = XShmCreateImage (xcontext->disp, xcontext->visual,
-      xcontext->depth, ZPixmap, NULL, &ximage->SHMInfo, 1, 1);
-  if (!ximage->ximage) {
-    GST_WARNING ("could not XShmCreateImage a 1x1 image");
-    goto beach;
-  }
-  ximage->size = ximage->ximage->height * ximage->ximage->bytes_per_line;
-
-  ximage->SHMInfo.shmid = shmget (IPC_PRIVATE, ximage->size, IPC_CREAT | 0777);
-  if (ximage->SHMInfo.shmid == -1) {
-    GST_WARNING ("could not get shared memory of %d bytes", ximage->size);
-    goto beach;
-  }
-
-  ximage->SHMInfo.shmaddr = shmat (ximage->SHMInfo.shmid, 0, 0);
-  if (ximage->SHMInfo.shmaddr == ((void *) -1)) {
-    GST_WARNING ("Failed to shmat: %s", g_strerror (errno));
-    goto beach;
-  }
-
-  ximage->ximage->data = ximage->SHMInfo.shmaddr;
-  ximage->SHMInfo.readOnly = FALSE;
-
-  if (XShmAttach (xcontext->disp, &ximage->SHMInfo) == 0) {
-    GST_WARNING ("Failed to XShmAttach");
-    goto beach;
-  }
-
-  XSync (xcontext->disp, 0);
-
-  XShmDetach (xcontext->disp, &ximage->SHMInfo);
-  XSync (xcontext->disp, FALSE);
-
-  shmdt (ximage->SHMInfo.shmaddr);
-  shmctl (ximage->SHMInfo.shmid, IPC_RMID, 0);
-
-  /* To be sure, reset the SHMInfo entry */
-  ximage->SHMInfo.shmaddr = ((void *) -1);
-
-  /* store whether we succeeded in result and reset error_caught */
-  result = !error_caught;
-  error_caught = FALSE;
-
-beach:
-  XSetErrorHandler (handler);
-  if (ximage->ximage)
-    XFree (ximage->ximage);
-  g_free (ximage);
-  XSync (xcontext->disp, FALSE);
-  return result;
-#endif /* HAVE_XSHM */
-}
-
 /* This function handles GstXImage creation depending on XShm availability */
 static GstXImage *
 gst_ximagesink_ximage_new (GstXImageSink * ximagesink, gint width, gint height)
 {
   GstXImage *ximage = NULL;
-  gboolean succeeded = FALSE;
 
   g_return_val_if_fail (GST_IS_XIMAGESINK (ximagesink), NULL);
   GST_DEBUG_OBJECT (ximagesink, "creating %dx%d", width, height);
 
-  ximage = g_new0 (GstXImage, 1);
-
-  ximage->width = width;
-  ximage->height = height;
-  ximage->ximagesink = ximagesink;
-
   g_mutex_lock (ximagesink->x_lock);
 
-#ifdef HAVE_XSHM
-  if (ximagesink->xcontext->use_xshm) {
-    ximage->ximage = XShmCreateImage (ximagesink->xcontext->disp,
-        ximagesink->xcontext->visual,
-        ximagesink->xcontext->depth,
-        ZPixmap, NULL, &ximage->SHMInfo, ximage->width, ximage->height);
-    if (!ximage->ximage) {
-      GST_ELEMENT_ERROR (ximagesink, RESOURCE, WRITE, (NULL),
-          ("could not XShmCreateImage a %dx%d image"));
-      goto beach;
-    }
+  ximage =
+      ximageutil_ximage_new (GST_ELEMENT (ximagesink), ximagesink->xcontext,
+      width, height);
 
-    /* we have to use the returned bytes_per_line for our shm size */
-    ximage->size = ximage->ximage->bytes_per_line * ximage->ximage->height;
-    GST_DEBUG_OBJECT (ximagesink, "XShm image size is %d, width %d, stride %d",
-        ximage->size, ximage->width, ximage->ximage->bytes_per_line);
-
-    ximage->SHMInfo.shmid = shmget (IPC_PRIVATE, ximage->size,
-        IPC_CREAT | 0777);
-    if (ximage->SHMInfo.shmid == -1) {
-      GST_ELEMENT_ERROR (ximagesink, RESOURCE, WRITE, (NULL),
-          ("could not get shared memory of %d bytes", ximage->size));
-      goto beach;
-    }
-
-    ximage->SHMInfo.shmaddr = shmat (ximage->SHMInfo.shmid, 0, 0);
-    if (ximage->SHMInfo.shmaddr == ((void *) -1)) {
-      GST_ELEMENT_ERROR (ximagesink, RESOURCE, WRITE, (NULL),
-          ("Failed to shmat: %s", g_strerror (errno)));
-      goto beach;
-    }
-
-    ximage->ximage->data = ximage->SHMInfo.shmaddr;
-    ximage->SHMInfo.readOnly = FALSE;
-
-    if (XShmAttach (ximagesink->xcontext->disp, &ximage->SHMInfo) == 0) {
-      GST_ELEMENT_ERROR (ximagesink, RESOURCE, WRITE, (NULL),
-          ("Failed to XShmAttach"));
-      goto beach;
-    }
-
-    XSync (ximagesink->xcontext->disp, FALSE);
-  } else
-#endif /* HAVE_XSHM */
-  {
-    ximage->ximage = XCreateImage (ximagesink->xcontext->disp,
-        ximagesink->xcontext->visual,
-        ximagesink->xcontext->depth,
-        ZPixmap, 0, NULL,
-        ximage->width, ximage->height, ximagesink->xcontext->bpp, 0);
-    if (!ximage->ximage) {
-      GST_ELEMENT_ERROR (ximagesink, RESOURCE, WRITE, (NULL),
-          ("could not XCreateImage a %dx%d image"));
-      goto beach;
-    }
-
-    /* we have to use the returned bytes_per_line for our image size */
-    ximage->size = ximage->ximage->bytes_per_line * ximage->ximage->height;
-    ximage->ximage->data = g_malloc (ximage->size);
-
-    XSync (ximagesink->xcontext->disp, FALSE);
-  }
-  succeeded = TRUE;
   g_mutex_unlock (ximagesink->x_lock);
 
-beach:
-  if (!succeeded) {
-    gst_ximagesink_ximage_destroy (ximagesink, ximage);
-    ximage = NULL;
-  }
-
   return ximage;
 }
 
@@ -281,31 +122,9 @@ gst_ximagesink_ximage_destroy (GstXImageSink * ximagesink, GstXImage * ximage)
 
   g_mutex_lock (ximagesink->x_lock);
 
-#ifdef HAVE_XSHM
-  if (ximagesink->xcontext->use_xshm) {
-    if (ximage->SHMInfo.shmaddr != ((void *) -1)) {
-      XShmDetach (ximagesink->xcontext->disp, &ximage->SHMInfo);
-      XSync (ximagesink->xcontext->disp, 0);
-      shmdt (ximage->SHMInfo.shmaddr);
-    }
-    if (ximage->SHMInfo.shmid > 0)
-      shmctl (ximage->SHMInfo.shmid, IPC_RMID, 0);
-    if (ximage->ximage)
-      XDestroyImage (ximage->ximage);
-
-  } else
-#endif /* HAVE_XSHM */
-  {
-    if (ximage->ximage) {
-      XDestroyImage (ximage->ximage);
-    }
-  }
-
-  XSync (ximagesink->xcontext->disp, FALSE);
+  ximageutil_ximage_destroy (ximagesink->xcontext, ximage);
 
   g_mutex_unlock (ximagesink->x_lock);
-
-  g_free (ximage);
 }
 
 /* This function puts a GstXImage on a GstXImageSink's window */
@@ -346,6 +165,7 @@ gst_ximagesink_ximage_put (GstXImageSink * ximagesink, GstXImage * ximage)
 
   XSync (ximagesink->xcontext->disp, FALSE);
 
+  ximagesink->need_expose = FALSE;
   g_mutex_unlock (ximagesink->x_lock);
 }
 
@@ -675,61 +495,6 @@ gst_ximagesink_handle_xevents (GstXImageSink * ximagesink, GstPad * pad)
   g_mutex_unlock (ximagesink->x_lock);
 }
 
-/* This function calculates the pixel aspect ratio based on the properties
- * in the xcontext structure and stores it there. */
-static void
-gst_ximagesink_calculate_pixel_aspect_ratio (GstXContext * xcontext)
-{
-  gint par[][2] = {
-    {1, 1},                     /* regular screen */
-    {16, 15},                   /* PAL TV */
-    {11, 10},                   /* 525 line Rec.601 video */
-    {54, 59}                    /* 625 line Rec.601 video */
-  };
-  gint i;
-  gint index;
-  gdouble ratio;
-  gdouble delta;
-
-#define DELTA(idx) (ABS (ratio - ((gdouble) par[idx][0] / par[idx][1])))
-
-  /* first calculate the "real" ratio based on the X values;
-   * which is the "physical" w/h divided by the w/h in pixels of the display */
-  ratio = (gdouble) (xcontext->widthmm * xcontext->height)
-      / (xcontext->heightmm * xcontext->width);
-
-  /* DirectFB's X in 720x576 reports the physical dimensions wrong, so
-   * override here */
-  if (xcontext->width == 720 && xcontext->height == 576) {
-    ratio = 4.0 * 576 / (3.0 * 720);
-  }
-  GST_DEBUG ("calculated pixel aspect ratio: %f", ratio);
-
-  /* now find the one from par[][2] with the lowest delta to the real one */
-  delta = DELTA (0);
-  index = 0;
-
-  for (i = 1; i < sizeof (par) / (sizeof (gint) * 2); ++i) {
-    gdouble this_delta = DELTA (i);
-
-    if (this_delta < delta) {
-      index = i;
-      delta = this_delta;
-    }
-  }
-
-  GST_DEBUG ("Decided on index %d (%d/%d)", index,
-      par[index][0], par[index][1]);
-
-  g_free (xcontext->par);
-  xcontext->par = g_new0 (GValue, 1);
-  g_value_init (xcontext->par, GST_TYPE_FRACTION);
-  gst_value_set_fraction (xcontext->par, par[index][0], par[index][1]);
-  GST_DEBUG ("set xcontext PAR to %d/%d",
-      gst_value_get_fraction_numerator (xcontext->par),
-      gst_value_get_fraction_denominator (xcontext->par));
-}
-
 /* This function gets the X Display and global info about it. Everything is
    stored in our object and will be cleaned when the object is disposed. Note
    here that caps for supported format are generated without any window or
@@ -738,91 +503,22 @@ static GstXContext *
 gst_ximagesink_xcontext_get (GstXImageSink * ximagesink)
 {
   GstXContext *xcontext = NULL;
-  XPixmapFormatValues *px_formats = NULL;
-  gint nb_formats = 0, i;
 
   g_return_val_if_fail (GST_IS_XIMAGESINK (ximagesink), NULL);
 
-  xcontext = g_new0 (GstXContext, 1);
-
   g_mutex_lock (ximagesink->x_lock);
 
-  xcontext->disp = XOpenDisplay (ximagesink->display_name);
+  xcontext =
+      ximageutil_xcontext_get (GST_ELEMENT (ximagesink),
+      ximagesink->display_name);
 
-  if (!xcontext->disp) {
+  if (!xcontext) {
     g_mutex_unlock (ximagesink->x_lock);
-    g_free (xcontext);
     GST_ELEMENT_ERROR (ximagesink, RESOURCE, WRITE, (NULL),
         ("Could not open display"));
     return NULL;
   }
 
-  xcontext->screen = DefaultScreenOfDisplay (xcontext->disp);
-  xcontext->screen_num = DefaultScreen (xcontext->disp);
-  xcontext->visual = DefaultVisual (xcontext->disp, xcontext->screen_num);
-  xcontext->root = DefaultRootWindow (xcontext->disp);
-  xcontext->white = XWhitePixel (xcontext->disp, xcontext->screen_num);
-  xcontext->black = XBlackPixel (xcontext->disp, xcontext->screen_num);
-  xcontext->depth = DefaultDepthOfScreen (xcontext->screen);
-
-  xcontext->width = DisplayWidth (xcontext->disp, xcontext->screen_num);
-  xcontext->height = DisplayHeight (xcontext->disp, xcontext->screen_num);
-  xcontext->widthmm = DisplayWidthMM (xcontext->disp, xcontext->screen_num);
-  xcontext->heightmm = DisplayHeightMM (xcontext->disp, xcontext->screen_num);
-
-  GST_DEBUG_OBJECT (ximagesink, "X reports %dx%d pixels and %d mm x %d mm",
-      xcontext->width, xcontext->height, xcontext->widthmm, xcontext->heightmm);
-
-  gst_ximagesink_calculate_pixel_aspect_ratio (xcontext);
-
-  /* We get supported pixmap formats at supported depth */
-  px_formats = XListPixmapFormats (xcontext->disp, &nb_formats);
-
-  if (!px_formats) {
-    XCloseDisplay (xcontext->disp);
-    g_mutex_unlock (ximagesink->x_lock);
-    g_free (xcontext);
-    return NULL;
-  }
-
-  /* We get bpp value corresponding to our running depth */
-  for (i = 0; i < nb_formats; i++) {
-    if (px_formats[i].depth == xcontext->depth)
-      xcontext->bpp = px_formats[i].bits_per_pixel;
-  }
-
-  XFree (px_formats);
-
-  xcontext->endianness =
-      (ImageByteOrder (xcontext->disp) ==
-      LSBFirst) ? G_LITTLE_ENDIAN : G_BIG_ENDIAN;
-
-#ifdef HAVE_XSHM
-  /* Search for XShm extension support */
-  if (XShmQueryExtension (xcontext->disp) &&
-      gst_ximagesink_check_xshm_calls (xcontext)) {
-    xcontext->use_xshm = TRUE;
-    GST_DEBUG ("ximagesink is using XShm extension");
-  } else {
-    xcontext->use_xshm = FALSE;
-    GST_DEBUG ("ximagesink is not using XShm extension");
-  }
-#endif /* HAVE_XSHM */
-
-  /* our caps system handles 24/32bpp RGB as big-endian. */
-  if ((xcontext->bpp == 24 || xcontext->bpp == 32) &&
-      xcontext->endianness == G_LITTLE_ENDIAN) {
-    xcontext->endianness = G_BIG_ENDIAN;
-    xcontext->visual->red_mask = GUINT32_TO_BE (xcontext->visual->red_mask);
-    xcontext->visual->green_mask = GUINT32_TO_BE (xcontext->visual->green_mask);
-    xcontext->visual->blue_mask = GUINT32_TO_BE (xcontext->visual->blue_mask);
-    if (xcontext->bpp == 24) {
-      xcontext->visual->red_mask >>= 8;
-      xcontext->visual->green_mask >>= 8;
-      xcontext->visual->blue_mask >>= 8;
-    }
-  }
-
   /* update object's par with calculated one if not set yet */
   if (!ximagesink->par) {
     ximagesink->par = g_new0 (GValue, 1);
@@ -838,7 +534,7 @@ gst_ximagesink_xcontext_get (GstXImageSink * ximagesink)
       "blue_mask", G_TYPE_INT, xcontext->visual->blue_mask,
       "width", GST_TYPE_INT_RANGE, 1, G_MAXINT,
       "height", GST_TYPE_INT_RANGE, 1, G_MAXINT,
-      "framerate", GST_TYPE_DOUBLE_RANGE, 1.0, 100.0, NULL);
+      "framerate", GST_TYPE_DOUBLE_RANGE, 1.0, 1000.0, NULL);
   if (ximagesink->par) {
     int nom, den;
 
@@ -861,18 +557,15 @@ gst_ximagesink_xcontext_clear (GstXImageSink * ximagesink)
   g_return_if_fail (GST_IS_XIMAGESINK (ximagesink));
   g_return_if_fail (ximagesink->xcontext != NULL);
 
-  gst_caps_free (ximagesink->xcontext->caps);
-  g_free (ximagesink->xcontext->par);
   g_free (ximagesink->par);
   ximagesink->par = NULL;
 
   g_mutex_lock (ximagesink->x_lock);
 
-  XCloseDisplay (ximagesink->xcontext->disp);
+  ximageutil_xcontext_clear (ximagesink->xcontext);
 
   g_mutex_unlock (ximagesink->x_lock);
 
-  g_free (ximagesink->xcontext);
   ximagesink->xcontext = NULL;
 }
 
@@ -1091,19 +784,72 @@ gst_ximagesink_change_state (GstElement * element)
   return GST_STATE_SUCCESS;
 }
 
+static void
+gst_ximagesink_handle_event (GstXImageSink * ximagesink, GstEvent * event,
+    GstPad * pad)
+{
+  switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_FILLER:
+    {
+      GstClockTime ts = GST_EVENT_TIMESTAMP (event);
+      GstClockTime dur = gst_event_filler_get_duration (event);
+      GstClockTimeDiff step;
+
+      if (!GST_CLOCK_TIME_IS_VALID (ts)) {
+        ts = ximagesink->time;
+      }
+      if (GST_CLOCK_TIME_IS_VALID (dur)) {
+        ts += dur;
+      }
+
+      gst_event_unref (event);
+      event = NULL;
+
+      if (ximagesink->framerate > 0)
+        step = GST_SECOND / ximagesink->framerate;
+      else
+        step = GST_CLOCK_DIFF (ts, ximagesink->time);
+
+      /*
+       * Advance the clock to the timestamp of the event, in steps
+       * at the framerate so we send navigation events and redraw nicely.
+       */
+      if (step > 0 && GST_VIDEOSINK_CLOCK (ximagesink)) {
+        while (ximagesink->time < ts) {
+          GstClockTime next_ts = MIN (ximagesink->time + step, ts);
+
+          if (next_ts > ximagesink->time) {
+            gst_element_wait (GST_ELEMENT (ximagesink), next_ts);
+          }
+          ximagesink->time = next_ts;
+
+          g_mutex_lock (ximagesink->stream_lock);
+          gst_ximagesink_handle_xevents (ximagesink, pad);
+          g_mutex_unlock (ximagesink->stream_lock);
+          gst_ximagesink_send_pending_navigation (ximagesink);
+        }
+      }
+
+      break;
+    }
+    default:
+      gst_pad_event_default (pad, event);
+  }
+}
+
 static void
 gst_ximagesink_chain (GstPad * pad, GstData * data)
 {
-  GstBuffer *buf = GST_BUFFER (data);
+  GstBuffer *buf;
   GstXImageSink *ximagesink;
 
   g_return_if_fail (GST_IS_PAD (pad));
-  g_return_if_fail (buf != NULL);
+  g_return_if_fail (data != NULL);
 
   ximagesink = GST_XIMAGESINK (gst_pad_get_parent (pad));
 
   if (GST_IS_EVENT (data)) {
-    gst_pad_event_default (pad, GST_EVENT (data));
+    gst_ximagesink_handle_event (ximagesink, GST_EVENT (data), pad);
     return;
   }
 
@@ -1157,9 +903,9 @@ gst_ximagesink_chain (GstPad * pad, GstData * data)
   gst_buffer_unref (buf);
 
   gst_ximagesink_handle_xevents (ximagesink, pad);
-  gst_ximagesink_send_pending_navigation (ximagesink);
 
   g_mutex_unlock (ximagesink->stream_lock);
+  gst_ximagesink_send_pending_navigation (ximagesink);
 }
 
 /* Buffer management */
@@ -1172,8 +918,8 @@ gst_ximagesink_buffer_free (GstBuffer * buffer)
 
   ximage = GST_BUFFER_PRIVATE (buffer);
 
-  g_assert (GST_IS_XIMAGESINK (ximage->ximagesink));
-  ximagesink = ximage->ximagesink;
+  g_assert (GST_IS_XIMAGESINK (ximage->parent));
+  ximagesink = GST_XIMAGESINK (ximage->parent);
 
   /* If our geometry changed we can't reuse that image. */
   if ((ximage->width != GST_VIDEOSINK_WIDTH (ximagesink)) ||
@@ -1236,7 +982,7 @@ gst_ximagesink_buffer_alloc (GstPad * pad, guint64 offset, guint size)
     /* Storing some pointers in the buffer */
     GST_BUFFER_PRIVATE (buffer) = ximage;
 
-    GST_BUFFER_DATA (buffer) = ximage->ximage->data;
+    GST_BUFFER_DATA (buffer) = (guint8 *) ximage->ximage->data;
     GST_BUFFER_FREE_DATA_FUNC (buffer) = gst_ximagesink_buffer_free;
     GST_BUFFER_SIZE (buffer) = ximage->size;
     return buffer;
@@ -1295,6 +1041,7 @@ gst_ximagesink_send_pending_navigation (GstXImageSink * ximagesink)
          to match the applied scaling. So here we just add the offset if the image
          is centered in the window.  */
 
+      g_mutex_lock (ximagesink->stream_lock);
       x_offset = ximagesink->xwindow->width - GST_VIDEOSINK_WIDTH (ximagesink);
       y_offset =
           ximagesink->xwindow->height - GST_VIDEOSINK_HEIGHT (ximagesink);
@@ -1307,6 +1054,9 @@ gst_ximagesink_send_pending_navigation (GstXImageSink * ximagesink)
         y -= y_offset / 2;
         gst_structure_set (structure, "pointer_y", G_TYPE_DOUBLE, y, NULL);
       }
+      g_mutex_unlock (ximagesink->stream_lock);
+
+      GST_DATA_FLAG_UNSET (event, GST_EVENT_COMMON_FLAG_NEED_RESPONSE);
 
       gst_pad_send_event (gst_pad_get_peer (GST_VIDEOSINK_PAD (ximagesink)),
           event);
@@ -1471,6 +1221,12 @@ gst_ximagesink_expose (GstXOverlay * overlay)
   if (!ximagesink->xwindow)
     return;
 
+  /* Set the need_expose flag before we wait for the lock. 
+   * If it's clear by the time we get to put the image,
+   * the stream beat us to it so don't bother
+   */
+  ximagesink->need_expose = TRUE;
+
   /* We don't want chain to iterate while we do that. We might act on random
      cur_image and different geometry */
   g_mutex_lock (ximagesink->stream_lock);
@@ -1484,7 +1240,7 @@ gst_ximagesink_expose (GstXOverlay * overlay)
 
   gst_ximagesink_xwindow_clear (ximagesink, ximagesink->xwindow);
 
-  if (ximagesink->cur_image)
+  if (ximagesink->need_expose && ximagesink->cur_image)
     gst_ximagesink_ximage_put (ximagesink, ximagesink->cur_image);
 
   g_mutex_unlock (ximagesink->stream_lock);
@@ -1655,6 +1411,7 @@ gst_ximagesink_init (GstXImageSink * ximagesink)
 
   ximagesink->sw_scaling_failed = FALSE;
   ximagesink->synchronous = FALSE;
+  ximagesink->need_expose = FALSE;
 
   ximagesink->par = NULL;
 
@@ -1703,6 +1460,9 @@ gst_ximagesink_class_init (GstXImageSinkClass * klass)
   gobject_class->get_property = gst_ximagesink_get_property;
 
   gstelement_class->change_state = gst_ximagesink_change_state;
+
+  GST_DEBUG_CATEGORY_INIT (gst_debug_ximagesink, "ximagesink", 0,
+      "ximagesink element");
 }
 
 /* ============================================================= */
@@ -1763,26 +1523,3 @@ gst_ximagesink_get_type (void)
 
   return ximagesink_type;
 }
-
-static gboolean
-plugin_init (GstPlugin * plugin)
-{
-  /* Loading the library containing GstVideoSink, our parent object */
-  if (!gst_library_load ("gstvideo"))
-    return FALSE;
-
-  if (!gst_element_register (plugin, "ximagesink",
-          GST_RANK_SECONDARY, GST_TYPE_XIMAGESINK))
-    return FALSE;
-
-  GST_DEBUG_CATEGORY_INIT (gst_debug_ximagesink, "ximagesink", 0,
-      "ximagesink element");
-
-  return TRUE;
-}
-
-GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
-    GST_VERSION_MINOR,
-    "ximagesink",
-    "XFree86 video output plugin based on standard Xlib calls",
-    plugin_init, VERSION, GST_LICENSE, GST_PACKAGE, GST_ORIGIN)
