@@ -22,18 +22,7 @@
 
 #include <gst/video/videosink.h>
 
-#ifdef HAVE_XSHM
-#include <sys/types.h>
-#include <sys/ipc.h>
-#include <sys/shm.h>
-#endif /* HAVE_XSHM */
-
-#include <X11/Xlib.h>
-#include <X11/Xutil.h>
-
-#ifdef HAVE_XSHM
-#include <X11/extensions/XShm.h>
-#endif /* HAVE_XSHM */
+#include "ximageutil.h"
 
 #include <string.h>
 #include <math.h>
@@ -51,61 +40,9 @@ G_BEGIN_DECLS
 #define GST_IS_XIMAGESINK_CLASS(obj) \
   (G_TYPE_CHECK_CLASS_TYPE((klass), GST_TYPE_XIMAGESINK))
 
-typedef struct _GstXContext GstXContext;
-typedef struct _GstXWindow GstXWindow;
-typedef struct _GstXImage GstXImage;
-
 typedef struct _GstXImageSink GstXImageSink;
 typedef struct _GstXImageSinkClass GstXImageSinkClass;
 
-/* Global X Context stuff */
-struct _GstXContext {
-  Display *disp;
-
-  Screen *screen;
-  gint screen_num;
-
-  Visual *visual;
-
-  Window root;
-
-  gulong white, black;
-
-  gint depth;
-  gint bpp;
-  gint endianness;
-
-  gint width, height;
-  gint widthmm, heightmm;
-  GValue *par;                  /* calculated pixel aspect ratio */
-
-  gboolean use_xshm;
-
-  GstCaps *caps;
-};
-
-/* XWindow stuff */
-struct _GstXWindow {
-  Window win;
-  gint width, height;
-  gboolean internal;
-  GC gc;
-};
-
-/* XImage stuff */
-struct _GstXImage {
-  /* Reference to the ximagesink we belong to */
-  GstXImageSink *ximagesink;
-
-  XImage *ximage;
-
-#ifdef HAVE_XSHM
-  XShmSegmentInfo SHMInfo;
-#endif /* HAVE_XSHM */
-
-  gint width, height, size;
-};
-
 struct _GstXImageSink {
   /* Our element stuff */
   GstVideoSink videosink;
